/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "HeaderDialog.h"

HeaderDialog::HeaderDialog(SaveData *saveData, QWidget *parent, int viewType)
	: QDialog(parent, Qt::Dialog | Qt::WindowCloseButtonHint), saveData(saveData)
{
	setWindowTitle(tr("Proprits"));

	group1 = new QGroupBox(tr("Memory card header"), this);

	exists_lbl = new QLabel(tr("Existe :"));
	exists = new QCheckBox;

	QLabel *country_lbl = new QLabel(tr("Rgion :"));
	country = new QComboBox;
	country->addItem(QIcon(":/images/jp.png"), tr("Japon"), QVariant('I'));
	country->addItem(QIcon(":/images/us.png"), tr("Amrique"), QVariant('A'));
	country->addItem(QIcon(":/images/eu.png"), tr("Europe"), QVariant('E'));
	country->addItem(tr("Invalide"), QVariant('\x00'));

	QLabel *code_lbl = new QLabel(tr("Code :"));
	code = new QComboBox;
	code->setEditable(true);
	code->lineEdit()->setMaxLength(10);
	fill_code(code);

	id_lbl = new QLabel(tr("Identifiant :"));
	id = new QComboBox;
	id->setEditable(true);
	id->lineEdit()->setMaxLength(8);
	fill_id(id);

	QGridLayout *layout1 = new QGridLayout(group1);
	layout1->addWidget(exists_lbl, 0, 0);
	layout1->addWidget(exists, 0, 1);
	layout1->addWidget(country_lbl, 1, 0);
	layout1->addWidget(country, 1, 1);
	layout1->addWidget(code_lbl, 2, 0);
	layout1->addWidget(code, 2, 1);
	layout1->addWidget(id_lbl, 3, 0);
	layout1->addWidget(id, 3, 1);

	group2 = new QGroupBox(tr("Slot header"), this);

	desc = new QLabel();

	QLabel *bloc_lbl = new QLabel(tr("Blocs utiliss :"));
	bloc = new QLabel();

	QLabel *desc_lbl = new QLabel(tr("Description :"));

	QLabel *icon1_lbl = new QLabel(tr("Icne :"));
	icon1 = new QLabel();
	QPushButton *icon1_saveButton = new QPushButton();
	icon1_saveButton->setIcon(QIcon(":/images/save.png"));
	icon1_saveButton->setFlat(true);
	icon2_lbl = new QLabel(tr("Icne additionnel :"));
	icon2 = new QLabel();
	icon2_saveButton = new QPushButton();
	icon2_saveButton->setIcon(QIcon(":/images/save.png"));
	icon2_saveButton->setFlat(true);

	QGridLayout *layout2 = new QGridLayout(group2);
	layout2->addWidget(bloc_lbl, 0, 0);
	layout2->addWidget(bloc, 0, 1, 1, 2);
	layout2->addWidget(desc_lbl, 1, 0);
	layout2->addWidget(desc, 1, 1, 1, 2);
	layout2->addWidget(icon1_lbl, 2, 0);
	layout2->addWidget(icon1, 2, 1);
	layout2->addWidget(icon1_saveButton, 2, 2, Qt::AlignRight);
	layout2->addWidget(icon2_lbl, 3, 0);
	layout2->addWidget(icon2, 3, 1);
	layout2->addWidget(icon2_saveButton, 3, 2, Qt::AlignRight);

	QPushButton *buttonSave = new QPushButton(tr("Ok"));
	QPushButton *button = new QPushButton(tr("Fermer"));
	connect(buttonSave, SIGNAL(released()), SLOT(save()));
	connect(button, SIGNAL(released()), SLOT(close()));
	connect(icon1_saveButton, SIGNAL(released()), SLOT(saveIcon1()));
	connect(icon2_saveButton, SIGNAL(released()), SLOT(saveIcon2()));

	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(group1, 0, 0, 1, 2);
	layout->addWidget(group2, 1, 0, 1, 2);
	layout->setRowStretch(2, 1);
	layout->addWidget(buttonSave, 3, 0, Qt::AlignRight);
	layout->addWidget(button, 3, 1, Qt::AlignLeft);

	fill(viewType);
}

void HeaderDialog::fill_code(QComboBox *code)
{
	code->addItem("SLPSP01880");
	code->addItem("SLUSP00892");
	code->addItem("SLESP02080");
	code->addItem("SLESP02081");
	code->addItem("SLESP02082");
	code->addItem("SLESP02083");
	code->addItem("SLPMP87384");
}

void HeaderDialog::fill_id(QComboBox *id)
{
	id->addItem("FF0800");
	id->addItem("042600");
	id->addItem("052000");
}

void HeaderDialog::fill(int viewType)
{
	setWindowIcon(QIcon(saveData->icon()));

	if(viewType == NORMALVIEW && !saveData->hasMC_header()) {
		group1->hide();
	}
	else {
		// Fill group1 (MC_header)

		QByteArray MC_header = saveData->MC_header();

		if(viewType == RESTOREVIEW || viewType == CREATEVIEW) {
			exists->setChecked(true);
		}
		else {
			exists->setChecked(!saveData->isDelete());
			id->lineEdit()->setText(QString(MC_header.mid(22, 8)));
		}

		if(viewType != CREATEVIEW) {

			int index;
			if(MC_header.at(10)=='B' && (index=country->findData(MC_header.at(11))) != -1)
			{
				country->setCurrentIndex(index);
				if(index != 3)		country->setMaxCount(3);
			}
			else
			{
				if(viewType == RESTOREVIEW) {
					country->setCurrentIndex(0);
				}
				else {
					country->setItemData(3, MC_header.at(11));
					country->setCurrentIndex(3);//Invalid
				}
			}

			QString codestr = QString(MC_header.mid(12, 10));
			if(codestr.size()==10 && (index=code->findText(codestr)) != -1)
			{
				code->setCurrentIndex(index);
			}
			else if(viewType == NORMALVIEW) {
				code->setEditText(codestr);
			}
		}
	}

	if(viewType == RESTOREVIEW || viewType == CREATEVIEW) {
		group2->hide();

		exists->hide();
		exists_lbl->hide();
		id_lbl->hide();
		id->hide();
		country->setMaxCount(3);
	}
	else {
		// Fill group2 (SC_header)

		QString short_desc = saveData->getShortDescription();
		if(!short_desc.isEmpty()) {
			desc->setText(short_desc);
		}

		bloc->setText(QString::number((quint8)saveData->header().at(3)));
		icon1->setPixmap(saveData->icon());
		connect(saveData->saveIcon(), SIGNAL(nextIcon(QPixmap)), icon1, SLOT(setPixmap(QPixmap)));
		if(saveData->isFF8()) {
			icon2->setPixmap(saveData->icon(true));
		}
		else {
			icon2_lbl->hide();
			icon2->hide();
			icon2_saveButton->hide();
		}
	}
}

void HeaderDialog::save()
{
	char countryValue = (char)country->itemData(country->currentIndex()).toInt();
	QByteArray idValue, codeValue = code->currentText().leftJustified(10, '\x00', true).toLatin1();
	if(countryValue=='E')
		idValue = "0520";
	else if(countryValue=='A')
		idValue = "0426";
	else
		idValue = "FF08";
	idValue.append(QString("%1").arg(saveData->id(), 2, 10, QChar('0')).leftJustified(4, '\x00', true).toLatin1());

	QByteArray MC_header = saveData->MC_header();
	if(MC_header.isEmpty()) {
		MC_header.append("\x51\x00\x00\x00\x00\x20\x00\x00\xff\xff", 10);//Main header (10)
		MC_header.append('B');//1
		MC_header.append(countryValue);//1
		MC_header.append(codeValue);//10
		MC_header.append(idValue);//6
	}
	else {
		MC_header[0] = exists->isChecked() ? 0x51 : 0xa1;
		MC_header[10] = 'B';
		MC_header[11] = countryValue;

		MC_header.replace(12, 10, codeValue);
		if(id->isVisible())
			MC_header.replace(22, 8, id->currentText().leftJustified(8, '\x00', true).toLatin1());
		else
			MC_header.replace(22, 8, idValue);
	}

	saveData->setMC_header(MC_header);

	accept();
}

void HeaderDialog::saveIcon(bool chocobo_world_icon)
{
	QString path = Config::value("savePathIcon").isEmpty() ? QString() : Config::value("savePathIcon") % "/";
	path = QFileDialog::getSaveFileName(this, tr("Enregistrer sous"), path % QString("icon%1%2").arg(saveData->id()+1).arg(chocobo_world_icon ? "b" : ""), QString("Image PNG (*.png);;Image JPG (*.jpg *.jpeg);;Image BMP (*.bmp)"));
	if(path.isEmpty())		return;

	int index = path.lastIndexOf('/');
	Config::setValue("savePathIcon", index == -1 ? path : path.left(index));

	if(!saveData->icon(chocobo_world_icon).save(path))
		QMessageBox::warning(this, tr("Erreur"), tr("Format incorrect."));
}

void HeaderDialog::saveIcon1()
{
	saveIcon();
}

void HeaderDialog::saveIcon2()
{
	saveIcon(true);
}
