/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "DiversEditor.h"

DiversEditor::DiversEditor(QWidget *parent) : QWidget(parent), data(NULL)
{
	// buildWidget();
	builded = false;
}

void DiversEditor::load(MAIN *data, int freq_value)
{
	if(!builded)
		buildWidget();
	this->data = data;
	this->freq_value = freq_value;
	fillPage();
}

void DiversEditor::updateMode()
{
	unknownE->setVisible(Config::mode());
}

void DiversEditor::updateTime()
{
	if(data==NULL)  return;
	quint32 time = data->stats.game_time;
	tempsSecE->setMaximum(Config::freq(freq_value)-1);
	tempsSecE->setValue(Config::sec(time, freq_value));
	tempsMinE->setValue(Config::min(time, freq_value));
	tempsHourE->setMaximum(Config::freq(freq_value)==60 ? 1193046 : 1431655);
	tempsHourE->setValue(Config::hour(time, freq_value));
}

void DiversEditor::buildWidget()
{
	builded = true;
	QVBoxLayout *layout = new QVBoxLayout(this);

	tabWidget = new QTabWidget(this);
	tabWidget->addTab(buildPage1(), tr("Divers"));
	tabWidget->addTab(buildPage2(), tr("Tutoriel"));

	layout->addWidget(tabWidget);
	layout->setContentsMargins(QMargins());
}

QWidget *DiversEditor::buildPage1()
{
	QWidget *ret = new QWidget(this);

	QGridLayout *grid = new QGridLayout(ret);
	
	QGroupBox *partyGBE = new QGroupBox(tr("quipe"), ret);
	QGridLayout *partyGBL = new QGridLayout(partyGBE);
	partyGBL->addWidget(new QLabel(tr("Menus :")), 0, 0);
	partyGBL->addWidget(new QLabel(tr(" l'cran :")), 1, 0);
	int i, j;
	for(i=0 ; i<3 ; ++i)
	{
		partyE << new QComboBox(partyGBE);
		partyE.last()->addItem("-", QVariant(255));
		for(j=0 ; j<8 ; ++j)
			partyE.last()->addItem(QIcon(QString(":/images/icons/perso%1.png").arg(j)), Data::names.at(j), QVariant(j));
		for(j=8 ; j<255 ; ++j)
			partyE.last()->addItem(QString::number(j), QVariant(j));
		partyGBL->addWidget(partyE.last(), 0, i+1);
		
		partySortE << new QComboBox(partyGBE);
		partySortE.last()->addItem("-", QVariant(255));
		for(j=0 ; j<11 ; ++j)
			partySortE.last()->addItem(QIcon(QString(":/images/icons/perso%1.png").arg(j)), Data::names.at(j), QVariant(j));
		for(j=11 ; j<255 ; ++j)
			partySortE.last()->addItem(QString::number(j), QVariant(j));
		partyGBL->addWidget(partySortE.last(), 1, i+1);
	}
	
	dreamE = new QCheckBox(tr("Seule l'quipe principale est visible (rve avec Laguna)"), partyGBE);
	partyGBL->addWidget(dreamE, 2, 0, 1, 4, Qt::AlignLeft);
	
	QGroupBox *statsE = new QGroupBox(tr("Statistiques"), ret);
	
	argentE = new QDoubleSpinBox(statsE);
	argentE->setDecimals(0);
	argentE->setRange(0, MAX_INT32);

	tempsHourE = new QSpinBox(statsE);
	tempsMinE = new QSpinBox(statsE);
	tempsMinE->setMaximum(59);
	tempsSecE = new QSpinBox(statsE);

	QHBoxLayout *tempsL = new QHBoxLayout;
	tempsL->addWidget(tempsHourE);
	tempsL->addWidget(new QLabel(":",statsE));
	tempsL->addWidget(tempsMinE);
	tempsL->addWidget(new QLabel(":",statsE));
	tempsL->addWidget(tempsSecE);

	QHBoxLayout *argent_tempsL = new QHBoxLayout;
	argent_tempsL->addStretch();
	argent_tempsL->addWidget(new QLabel(tr("Temps :"),statsE));
	argent_tempsL->addLayout(tempsL);
	
	stepsE = new QDoubleSpinBox(statsE);
	stepsE->setDecimals(0);
	stepsE->setRange(0, MAX_INT32);

	battlewinE = new QDoubleSpinBox(statsE);
	battlewinE->setDecimals(0);
	battlewinE->setRange(0, MAX_INT32);
	battleescE = new QSpinBox(statsE);
	battleescE->setRange(0, MAX_INT16);
	
	seedLvlE = new QSpinBox(statsE);
	seedLvlE->setRange(0, 31);
	connect(seedLvlE, SIGNAL(valueChanged(int)), SLOT(nivS(int)));
	seedExpE = new QSpinBox(statsE);
	seedExpE->setRange(0, MAX_INT16);
	connect(seedExpE, SIGNAL(valueChanged(int)), SLOT(expS(int)));

	testSeedE = new QSpinBox(statsE);
	testSeedE->setRange(0, MAX_INT8);

	countdownE = new QDoubleSpinBox(statsE);
	countdownE->setDecimals(0);
	countdownE->setRange(0, MAX_INT32);
	
	QGridLayout *statsL = new QGridLayout(statsE);
	statsL->addWidget(new QLabel(tr("Argent :"),statsE), 0, 0);
	statsL->addWidget(argentE, 0, 1);
	statsL->addLayout(argent_tempsL, 0, 2, 1, 4);
	statsL->addWidget(new QLabel(tr("Niveau Seed :"),statsE), 1, 0);
	statsL->addWidget(seedLvlE, 1, 1);
	statsL->addWidget(new QLabel(tr("EXP Seed :"),statsE), 1, 2);
	statsL->addWidget(seedExpE, 1, 3);
	statsL->addWidget(new QLabel(tr("Niveau test Seed :"),statsE), 1, 4);
	statsL->addWidget(testSeedE, 1, 5);
	statsL->addWidget(new QLabel(tr("Nombre de pas :"),statsE), 2, 0);
	statsL->addWidget(stepsE, 2, 1);
	statsL->addWidget(new QLabel(tr("Combats gagns :"),statsE), 2, 2);
	statsL->addWidget(battlewinE, 2, 3);
	statsL->addWidget(new QLabel(tr("Combats fuis :"),statsE), 2, 4);
	statsL->addWidget(battleescE, 2, 5);
	statsL->addWidget(new QLabel(tr("Compte  rebours :"),statsE), 3, 0);
	statsL->addWidget(countdownE, 3, 1);

	unknownE = new QGroupBox(tr("Inconnu"), ret);
	unknown1E = new QDoubleSpinBox(statsE);
	unknown1E->setDecimals(0);
	unknown1E->setRange(0, MAX_INT32);
	unknown2E = new QSpinBox(unknownE);
	unknown2E->setRange(0, MAX_INT16);
	unknown3E = new QSpinBox(unknownE);
	unknown3E->setRange(0, MAX_INT16);
	unknown4E = new QDoubleSpinBox(statsE);
	unknown4E->setDecimals(0);
	unknown4E->setRange(0, MAX_INT32);
	QGridLayout *unknownL = new QGridLayout(unknownE);
	unknownL->addWidget(new QLabel(tr("Inconnu 1 :"),unknownE), 0, 0);
	unknownL->addWidget(unknown1E, 0, 1);
	unknownL->addWidget(new QLabel(tr("Inconnu 2 :"),unknownE), 0, 2);
	unknownL->addWidget(unknown2E, 0, 3);
	unknownL->addWidget(new QLabel(tr("Inconnu 3 :"),unknownE), 1, 0);
	unknownL->addWidget(unknown3E, 1, 1);
	unknownL->addWidget(new QLabel(tr("Inconnu 4 :"),unknownE), 1, 2);
	unknownL->addWidget(unknown4E, 1, 3);
	
	grid->addWidget(partyGBE, 0, 0, 1, 5);
	grid->addWidget(statsE, 1, 0, 1, 5);
	grid->addWidget(unknownE, 2, 0, 1, 5);
	
	grid->setRowStretch(3, 1);

	return ret;
}

QWidget *DiversEditor::buildPage2()
{
	QWidget *ret = new QWidget(this);

	QFont font;
	font.setPointSize(10);

	tutoE_list = new QTreeWidget(this);
	tutoE_list->setFont(font);
	tutoE_list->setHeaderLabel(tr("Catgorie"));
	tutoE_list->setIndentation(0);
	tutoE_list->setUniformRowHeights(true);

	QStringList tutoStrings;
	tutoStrings
			<< tr("Lieux/Mine de souffre") << tr("Lieux/Dollet") << tr("Lieux/Timber")
			<< tr("Lieux/Chane TV Timber") << tr("Lieux/Galbadia") << tr("Lieux/Tombe du roi inconnu")
			<< tr("Lieux/Winhill") << tr("Lieux/Prison du dsert") << tr("Lieux/Base des missiles")
			<< tr("Lieux/Aqueduc d'Horizon") << tr("Lieux/Horizon") << tr("Lieux/Trabia")
			<< tr("Lieux/Esthar(1)") << tr("Lieux/Esthar(2)") << tr("Lieux/Esthar(3)")
			<< tr("Lieux/Station balnaire") << tr("Lieux/Salt Lake") << tr("Lieux/Lune")
			<< tr("Lieux/Labo de Deep Sea") << tr("Lieux/Fouilles de Deep Sea") << tr("Termes/Compression Temporelle")
			<< tr("Termes/Origine de la BGU") << tr("Termes/Ondes hertziennes(1)") << tr("Termes/Ondes hertziennes(2)")
			<< tr("Termes/Larme slnite") << tr("Termes/Centra") << tr("Termes/Compteur Geyser")
			<< tr("Termes/Mages et sorcires") << tr("Termes/Pouvoirs magiques") << tr("Termes/Monolithe")
			<< tr("Termes/Lunatic Pandora") << tr("Termes/Tears Point") << tr("Termes/Sanctuaire d'Adel")
			<< tr("Termes/L'Hydre") << tr("Termes/Niveau MD") << tr("Termes/Refuge de Centra")
			<< tr("Termes/Timber Maniacs") << tr("Termes/Eyes On Me") << tr("Personnages/Dr. Geyser")
			<< tr("Personnages/Hyne") << tr("Personnages/Seeds Blancs") << tr("Personnages/Moomba")
			<< tr("Personnages/Tribu Shumi") << tr("Personnages/Chocobo") << tr("Personnages/Maire Dobe")
			<< tr("Zell") << tr("Quistis") << tr("Seifer")
			<< tr("Selphie") << tr("Linoa") << tr("Irvine")
			<< tr("Edea") << tr("Laguna") << tr("Kiros")
			<< tr("Ward") << tr("Linoa 2 (Canonisation)") << "???"
			<< "???" << tr("Termes/Rincarnation occulte") << tr("Rapport de combat")
			<< tr("Rapport Perso") << tr("Rapport G-Force") << tr("Actions/Association G-Force (inutilis)")
			<< tr("Actions/Association magique") << tr("Actions/Association lmentale") << tr("Actions/Association mentale")
			<< tr("Actions/Tutorial G-Forces") << tr("Actions/cran situation Squall") << tr("Actions/cran situation Zell")
			<< tr("Actions/cran situation Linoa") << tr("Actions/Intervertir") << "???"
			<< tr("Minimog") << tr("Rapport G-Force/Golgotha") << tr("Rapport G-Force/Shiva")
			<< tr("Rapport G-Force/Ifrit") << tr("Rapport G-Force/Ondine") << tr("Rapport G-Force/Taurus")
			<< tr("Rapport G-Force/Nosferatu") << tr("Rapport G-Force/Ahuri") << tr("Rapport G-Force/Leviathan")
			<< tr("Rapport G-Force/Zphyr") << tr("Rapport G-Force/Cerberus") << tr("Rapport G-Force/Alexander")
			<< tr("Rapport G-Force/Helltrain") << tr("Rapport G-Force/Bahamut") << tr("Rapport G-Force/Pampa")
			<< tr("Rapport G-Force/Tomberry") << tr("Rapport G-Force/Orbital") << tr("Cra-Mgi-Cl")
			<< tr("Cra-Mgi-Cl 2") << tr("Cra-Mgi-Gla") << tr("Cra-Mgi-Gla 2")
			<< tr("Cra-Mgi-Inc") << tr("Cra-Mgi-Inc 2") << tr("Cra-Mgi-Md")
			<< tr("Cra-Mgi-Md 2") << tr("Cra-Mgi-Temp") << tr("Cra-Mgi-Temp 2")
			<< tr("Cra-Mgi-Mtl") << tr("Cra-Mgi-Mtl 2") << tr("Cra-Mgi-Pro")
			<< tr("Cra-Mgi-Pro 2") << tr("Cra-Mgi-Tab") << tr("Cra-Mgi-Tab 2")
			<< tr("Cra-Mgi-Thr") << tr("Cra-Mgi-Thr 2") << tr("Cra-Mgi-Ana")
			<< tr("Cra-Mgi-Ana 2") << tr("Cra-balles") << tr("Cra-balles 2")
			<< tr("Cra-outils") << tr("Cra-outils 2") << tr("Cra-Thr-Tab")
			<< tr("Cra-Thr-Tab 2") << tr("Cra-Thr-GF") << tr("Cra-Thr-GF 2")
			<< tr("Cra-Capa-GF") << tr("Cra-Capa-GF 2") << tr("Cra-Mgi-Plus")
			<< tr("Cra-Mgi-Plus 2") << tr("Thr-Niv +") << tr("Thr-Niv + 2")
			<< tr("Freud") << tr("Cra-Mgi-Max") << tr("Cra-Mgi-Max 2")
			<< "???" << tr("Diplme Minotaure");

	QTreeWidgetItem *item;
	foreach(QString tutoString, tutoStrings) {
		item = new QTreeWidgetItem(QStringList(tutoString));
		item->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled);
		tutoE_list->addTopLevelItem(item);
	}

	QCheckBox *tutoCheckAll = new QCheckBox(tr("Slectionner tout"), ret);
	connect(tutoCheckAll, SIGNAL(toggled(bool)), SLOT(selectAll(bool)));

	QVBoxLayout *layout = new QVBoxLayout(ret);
	layout->addWidget(tutoE_list);
	layout->addWidget(tutoCheckAll, 0, Qt::AlignLeft);

	return ret;
}

void DiversEditor::selectAll(bool select)
{
	int count = tutoE_list->topLevelItemCount();
	for(int i=0 ; i<count ; ++i) {
		tutoE_list->topLevelItem(i)->setCheckState(0, select ? Qt::Checked : Qt::Unchecked);
	}
}

void DiversEditor::fillPage()
{
	argentE->setValue(data->misc.argent);
	
	seedExpE->setValue(data->fieldVars.seedExp);

	for(quint8 i=0 ; i<3 ; ++i)
	{
		partyE.at(i)->setCurrentIndex(partyE.at(i)->findData(QVariant(data->misc.party[i])));
		partySortE.at(i)->setCurrentIndex(partySortE.at(i)->findData(QVariant(data->stats.party[i])));
	}
	dreamE->setChecked(data->stats.dream & 1);
	
	stepsE->setValue(data->fieldVars.steps);
	battlewinE->setValue(data->stats.combats_gagnes);
	battleescE->setValue(data->stats.combats_fuis);

	testSeedE->setValue(data->stats.testLevel);
	countdownE->setValue(data->stats.countdown);

	unknown1E->setValue(data->misc.u1);
	unknown2E->setValue(data->misc.u2);
	unknown3E->setValue(data->misc.u3);
	unknown4E->setValue(data->misc.u4);

	int i, j;
	for(i=0 ; i<16 ; ++i) {
		for(j=0 ; j<8 ; ++j) {
			tutoE_list->topLevelItem(i*8+j)
					->setCheckState(0, (data->stats.tutorial_infos[i]>>j)&1 ? Qt::Checked : Qt::Unchecked);
		}
	}
}

void DiversEditor::savePage()
{
	data->misc.argent = argentE->value();
	data->stats.game_time = Config::time(tempsHourE->value(), tempsMinE->value(), tempsSecE->value(), freq_value);

	for(quint8 i=0 ; i<3 ; ++i)
	{
		data->misc.party[i] = partyE.at(i)->itemData(partyE.at(i)->currentIndex()).toUInt();
		data->stats.party[i] = partySortE.at(i)->itemData(partySortE.at(i)->currentIndex()).toUInt();
	}
	data->stats.dream = dreamE->isChecked() | (data->stats.dream & 0xFE);

	data->fieldVars.seedExp = seedExpE->value();
	
	data->fieldVars.steps = stepsE->value();
	data->stats.combats_gagnes = battlewinE->value();
	data->stats.combats_fuis = battleescE->value();

	data->stats.testLevel = testSeedE->value();

	data->stats.countdown = countdownE->value();

	data->misc.u1 = unknown1E->value();
	data->misc.u2 = unknown2E->value();
	data->misc.u3 = unknown3E->value();
	data->misc.u4 = unknown4E->value();

	int i, j;
	quint8 curTuto_info;
	for(i=0 ; i<16 ; ++i) {
		curTuto_info = 0;
		for(j=0 ; j<8 ; ++j) {
			curTuto_info |= (tutoE_list->topLevelItem(i*8+j)->checkState(0)==Qt::Checked)<<j;
		}
		data->stats.tutorial_infos[i] = curTuto_info;
	}
}

void DiversEditor::expS(int value)
{
	quint16 niv = value/100;
	if(seedLvlE->value() != niv)
	{
		seedLvlE->setValue(niv);
	}
}

void DiversEditor::nivS(int value)
{
	quint16 exp = value*100;
	if(exp > seedExpE->value() || exp + 100 <= seedExpE->value())
	{
		seedExpE->setValue(exp);
	}
}
