/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "GfEditor.h"

GfEditor::GfEditor(QWidget *parent) : QWidget(parent)
{
	// buildWidget();
	builded = false;
	this->id = 255;
}

void GfEditor::load(MAIN *data, bool jp)
{
	if(!builded)
		buildWidget();
	this->data = data;
	this->id = 255;
	this->jp = jp;
	fillPage();
}

void GfEditor::updateMode()
{
	inconnu1_labelE->setVisible(Config::mode());
	inconnu1_E->setVisible(Config::mode());
}

void GfEditor::buildWidget()
{
	builded = true;
	gfListe = new QListWidget(this);
	gfListe->setViewMode(QListView::IconMode);
	gfListe->setFlow(QListView::LeftToRight);
	gfListe->setIconSize(QSize(32,48));
	gfListe->setFixedHeight(57);
	gfListe->setWrapping(false);
	gfListe->setMovement(QListView::Static);
	gfListe->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	
	QListWidgetItem *item;
	for(quint8 i=0 ; i<17 ; ++i)
	{
		item = new QListWidgetItem(gfListe);
		item->setIcon(QIcon(QString(":/images/icons/gf%1.png").arg(i)));
		item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
	}
	gfListe->setCurrentRow(0);

	stackedWidget = new QStackedWidget(this);
	stackedWidget->addWidget(buildPage1());
	stackedWidget->addWidget(buildPage2());
	stackedWidget->setCurrentIndex(0);

	QVBoxLayout *layout = new QVBoxLayout(this);
	layout->setContentsMargins(QMargins());
	layout->addWidget(gfListe);
	layout->addWidget(stackedWidget);

	connect(gfListe, SIGNAL(currentRowChanged(int)), SLOT(fillPage(int)));
}

QWidget *GfEditor::buildPage1()
{
	QWidget *ret = new QWidget(this);
	
	existsE = new QCheckBox(ret);
	nameEdit = new QLineEdit(ret);
//	nameEdit->setMaxLength(11);
	hpEdit = new QSpinBox(ret);
	hpEdit->setRange(0, MAX_INT16);
	
	QHBoxLayout *mainEdit_l = new QHBoxLayout;
	mainEdit_l->addWidget(new QLabel(tr("Disponible"), ret));
	mainEdit_l->addWidget(existsE);
	mainEdit_l->addWidget(new QLabel(tr("Nom :"), ret));
	mainEdit_l->addWidget(nameEdit);
	mainEdit_l->addWidget(new QLabel(tr("HP actuels :"), ret));
	mainEdit_l->addWidget(hpEdit);
	
	expEdit = new QDoubleSpinBox(ret);
	expEdit->setDecimals(0);
	expEdit->setRange(0, MAX_INT32);
	connect(expEdit, SIGNAL(valueChanged(double)), SLOT(changeExp(double)));
	nivEdit = new QSpinBox(ret);
	nivEdit->setRange(1, 100);
	connect(nivEdit, SIGNAL(valueChanged(int)), SLOT(changeNiv(int)));
	
	killsEdit = new QSpinBox(ret);
	killsEdit->setRange(0, MAX_INT16);
	KOsEdit = new QSpinBox(ret);
	KOsEdit->setRange(0, MAX_INT16);
	
	inconnu1_labelE = new QLabel(tr("Inconnu :"), ret);
	inconnu1_E = new QSpinBox(ret);
	inconnu1_E->setRange(0, MAX_INT8);

	QGridLayout *statEditL = new QGridLayout;
	statEditL->addWidget(new QLabel(tr("Niveau :"), ret), 0, 0);
	statEditL->addWidget(nivEdit, 0, 1);
	statEditL->addWidget(new QLabel(tr("EXP :"), ret), 0, 2);
	statEditL->addWidget(expEdit, 0, 3);
	statEditL->addWidget(new QLabel(tr("Tus :"), ret), 1, 0);
	statEditL->addWidget(killsEdit, 1, 1);
	statEditL->addWidget(new QLabel(tr("Mort :"), ret), 1, 2);
	statEditL->addWidget(KOsEdit, 1, 3);
	statEditL->addWidget(inconnu1_labelE, 2, 0);
	statEditL->addWidget(inconnu1_E, 2, 1);
	
	liste = new QTreeWidget(ret);
	liste2 = new QTreeWidget(ret);
	liste->setHeaderLabels(QStringList() << QString() << tr("Capacits") << tr("PDC"));
	liste2->setHeaderLabels(QStringList() << QString() << tr("Capacits oublies"));
	liste->setIndentation(0);
	liste2->setIndentation(0);
	liste->setColumnHidden(0, true);
	liste2->setColumnHidden(0, true);
	liste->header()->setStretchLastSection(false);
	liste2->setFixedWidth(150);
	liste2->resizeColumnToContents(1);
	liste->setUniformRowHeights(true);
	liste2->setUniformRowHeights(true);
	
	addC = new QPushButton(tr("Ajouter"), ret);
	addC->setIcon(QIcon(":/images/plus.png"));
	connect(addC, SIGNAL(released()), SLOT(add_C()));
	
	removeC = new QPushButton(tr("Oublier"), ret);
	removeC->setIcon(QIcon(":/images/minus.png"));
	connect(removeC, SIGNAL(released()), SLOT(remove_C()));
	
	editC = new QPushButton(tr("Modifier"), ret);
	editC->setIcon(QIcon(":/images/edit.png"));
	connect(editC, SIGNAL(released()), SLOT(edit_C()));
	
	learnC = new QPushButton(tr("Apprendre"), ret);
	learnC->setIcon(QIcon(":/images/icons/learning.png"));
	connect(learnC, SIGNAL(released()), SLOT(changeLearning()));
	
	connect(liste, SIGNAL(itemSelectionChanged()), SLOT(enableButtons()));
	connect(liste, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), SLOT(edit_C(QTreeWidgetItem*)));
	
	QVBoxLayout *buttonC = new QVBoxLayout;
	buttonC->addWidget(addC);
	buttonC->addWidget(removeC);
	buttonC->addWidget(editC);
	buttonC->addWidget(learnC);
	buttonC->addStretch();
	
	restoreF = new QPushButton(tr("Restaurer"), ret);
	restoreF->setIcon(QIcon(":/images/restore.png"));
	connect(restoreF, SIGNAL(released()), SLOT(restore_C()));
	
	connect(liste2, SIGNAL(itemSelectionChanged()), SLOT(enableButtons2()));
	connect(liste2, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), SLOT(restore_C()));
	
	QGridLayout *grid = new QGridLayout(ret);
	grid->addLayout(mainEdit_l, 0, 0, 1, 4);
	grid->addLayout(statEditL, 1, 0, 1, 4);
	grid->addWidget(liste, 2, 0);
	grid->addLayout(buttonC, 2, 1);
	grid->addWidget(liste2, 2, 2);
	grid->addWidget(restoreF, 2, 3, Qt::AlignTop);
	
	return ret;
}

QWidget *GfEditor::buildPage2()
{
	QWidget *ret = new QWidget(this);

	QGroupBox *groupBox = new QGroupBox(tr("Divers"), ret);

	cronosE = new QLineEdit(groupBox);
//	cronosE->setMaxLength(11);
	odinE = new QCheckBox(tr("Odin"), groupBox);
	gilgameshE = new QCheckBox(tr("Gilgamesh"), groupBox);
	phoenixE = new QCheckBox(tr("Phnix"), groupBox);

	tombyE = new QSpinBox(groupBox);
	tombyE->setRange(0, MAX_INT16);
	tombySrE = new QSpinBox(groupBox);
	tombySrE->setRange(0, MAX_INT8);

	QGridLayout *grid = new QGridLayout(groupBox);
	grid->addWidget(new QLabel(Data::names.at(12)+tr(" :"),groupBox), 1, 0);
	grid->addWidget(cronosE, 1, 1);
	grid->addWidget(odinE, 1, 2);
	grid->addWidget(gilgameshE, 1, 3);
	grid->addWidget(phoenixE, 1, 4);
	grid->addWidget(new QLabel(tr("Tomberry tus :"),groupBox), 2, 0);
	grid->addWidget(tombyE, 2, 1);
	grid->addWidget(new QLabel(tr("Tomberry Sr vaincus :"),groupBox), 2, 2);
	grid->addWidget(tombySrE, 2, 3);
	grid->setRowStretch(3, 1);

	return ret;
}

void GfEditor::fillPage(int)
{
	savePage();
	this->id = gfListe->currentRow();

	if(id==16)
	{
		stackedWidget->setCurrentIndex(1);

		cronosE->setText(FF8Text::toString(data->misc.cronos, jp));
		odinE->setChecked((data->stats.dream >> 1) & 1);
		phoenixE->setChecked((data->stats.dream >> 2) & 1);
		gilgameshE->setChecked((data->stats.dream >> 3) & 1);

		tombyE->setValue(data->stats.tomberry_vaincus);
		tombySrE->setValue(data->stats.tomberry_sr_vaincu);
	}
	else
	{
		stackedWidget->setCurrentIndex(0);

		this->gf_data = &data->gfs[id];

		switch(id) {
		case 15:// Orbital
			this->nivSup = 1000;
			break;
		case 3:case 6:case 9:// Siren, Carbuncle, Cerberus
					this->nivSup = 400;
			break;
		default:
			this->nivSup = 500;
			break;
		}

		existsE->setChecked(gf_data->exists & 1);
		nameEdit->setText(FF8Text::toString(gf_data->name, jp));
		hpEdit->setValue(gf_data->HPs);

		expEdit->setValue(gf_data->exp);
		changeExp(gf_data->exp);

		killsEdit->setValue(gf_data->kills);
		KOsEdit->setValue(gf_data->KOs);

		inconnu1_E->setValue(gf_data->u1);

		quint8 abilityID = 0;
		QList<quint8> capacitesEnPlace;
		liste->clear();
		liste2->clear();

		//Listage des capacits aquises
		for(abilityID=0 ; abilityID<116 ; ++abilityID)
		{
			if(getCompleteAbility(abilityID))
			{
				addItem(abilityID, 1);

				capacitesEnPlace.append(abilityID);
			}
		}

		enableButtons();

		//Listage des capacits en apprentissage et des capacits oublies
		for(quint8 i=0 ; i<22 ; ++i)
		{
			abilityID = Data::innateAbilities[id][i];

			if(!capacitesEnPlace.contains(abilityID))//Si pas dj list prcdemment dans les capacits aquises
			{
				if(!((gf_data->forgotten >> i) & 1))//En apprentissage
				{
					addItem(abilityID, 0, i);
				}
				else //Oublies
				{
					addItem(abilityID, 2, i);
				}
			}
		}

		enableButtons2();

		liste->sortByColumn(0, Qt::AscendingOrder);
		liste2->sortByColumn(0, Qt::AscendingOrder);
		liste->header()->setResizeMode(1, QHeaderView::Stretch);
		liste->header()->setResizeMode(2, QHeaderView::ResizeToContents);
	}
}

void GfEditor::savePage()
{
	if(this->id==255)	return;

	const char *chaine;

	if(id==16) {
		chaine = FF8Text::toByteArray(cronosE->text(), jp).leftJustified(11, '\x00', true).append('\x00').constData();
		memcpy(&data->misc.cronos, chaine, 12);

		data->stats.dream = (odinE->isChecked() << 1) | (phoenixE->isChecked() << 2) | (gilgameshE->isChecked() << 3) | (data->stats.dream & 0xF1);
		data->stats.tomberry_vaincus = tombyE->value();
		data->stats.tomberry_sr_vaincu = tombySrE->value();
	}
	else {
		gf_data->exists = (quint8)existsE->isChecked();

		chaine = FF8Text::toByteArray(nameEdit->text(), jp).leftJustified(11, '\x00', true).append('\x00').constData();
		memcpy(&gf_data->name, chaine, 12);

		gf_data->exp = expEdit->value();
		gf_data->HPs = hpEdit->value();
		gf_data->kills = killsEdit->value();
		gf_data->KOs = KOsEdit->value();
		gf_data->u1 = inconnu1_E->value();
	}
}

QTreeWidgetItem *GfEditor::addItem(quint8 abilityID, quint8 type, quint8 innateAbID)
{
	QTreeWidgetItem *item;
	
	switch(type)
	{
	case 0://Capacits de la GF non acquises (vert=Apprentissage en cours, noir sinon)
		item = new QTreeWidgetItem(liste, QStringList() << QString("%1").arg(abilityID,3) << Data::abilities.at(abilityID) << QString("%1/%2").arg(gf_data->APs[innateAbID]).arg(Data::apsTab[abilityID]));
		if(gf_data->learning == abilityID)
		{
			item->setForeground(1, QBrush(Qt::darkGreen));
			item->setForeground(2, QBrush(Qt::darkGreen));
			item->setIcon(1, QIcon(":/images/icons/learning.png"));
		}
		else
		{
			item->setIcon(1, QIcon(QString(":/images/icons/capacity%1.png").arg(Data::abilityType(abilityID))));
		}
		break;
		
	case 1://Capacits acquises de la GF (noir) et capacits acquises en plus (gris)
		item = new QTreeWidgetItem(liste, QStringList() << QString("%1").arg(abilityID,3) << Data::abilities.at(abilityID) << tr("Acquis!"));
		if(posAbility(abilityID) == -1)
		{
			item->setForeground(1, QBrush(Qt::darkGray));
			item->setForeground(2, QBrush(Qt::darkGray));
			item->setIcon(1, QIcon(QString(":/images/icons/capacity%1.png").arg(Data::abilityType(abilityID))));
		}
		else if(gf_data->learning == abilityID)
		{
			item->setForeground(1, QBrush(Qt::darkGreen));
			item->setForeground(2, QBrush(Qt::darkGreen));
			item->setIcon(1, QIcon(":/images/icons/learning.png"));
		}
		else
		{
			item->setIcon(1, QIcon(QString(":/images/icons/capacity%1.png").arg(Data::abilityType(abilityID))));
		}
		break;

	default://Capacits oublies de la GF (rouge)
		item = new QTreeWidgetItem(liste2, QStringList() << QString("%1").arg(abilityID,3) << Data::abilities.at(abilityID));
		item->setForeground(1, QBrush(Qt::red));
		item->setIcon(1, QIcon(QString(":/images/icons/capacity%1.png").arg(Data::abilityType(abilityID))));
		break;
	}
	
	return item;
}

void GfEditor::remove_C()
{
	QList<QTreeWidgetItem *> selectedItems = liste->selectedItems();
	if(selectedItems.isEmpty())	return;
	quint8 abilityID = selectedItems.first()->text(0).toInt();

	delete selectedItems.first();
	
	if(abilityID == gf_data->learning)
	{
		gf_data->learning = 0;
	}
	
	qint8 pos;
	if((pos = posAbility(abilityID)) != -1)
	{
		QTreeWidgetItem *item = addItem(abilityID, 2, pos);
		gf_data->forgotten |= 1 << pos;
		liste2->sortByColumn(0, Qt::AscendingOrder);
		liste2->scrollToItem(item);
		liste2->setCurrentItem(item);
	}
	setCompleteAbility(abilityID, false);
}

void GfEditor::add_C()
{
	QDialog dialog(this);
	dialog.setWindowFlags(Qt::Dialog | Qt::WindowCloseButtonHint);
	
	QGridLayout *grid = new QGridLayout(&dialog);
	
	selection = new QComboBox(&dialog);
	Data::fillAbilities(selection);
	
	okC = new QPushButton(tr("Ajouter"), &dialog);
	okC->setEnabled(posAbility(1) == -1);
	okC->setIcon(QIcon(":/images/plus.png"));
	
	grid->addWidget(selection, 0, 0);
	grid->addWidget(okC, 1, 0);
	grid->setAlignment(okC, Qt::AlignHCenter);
	
	connect(okC, SIGNAL(released()), SLOT(addCapacity()));
	connect(selection, SIGNAL(currentIndexChanged(int)), SLOT(changeCapacity(int)));
	connect(okC, SIGNAL(released()), &dialog, SLOT(close()));	
	
	dialog.exec();
}

void GfEditor::edit_C(QTreeWidgetItem *item)
{
	if(posAbility(item->text(0).toInt()) != -1)
		edit_C();
}

void GfEditor::edit_C()
{
	QList<QTreeWidgetItem *> selectedItems = liste->selectedItems();
	if(selectedItems.isEmpty())	return;

	quint8 abilityID = selectedItems.first()->text(0).toInt();
	
	dialog = new QDialog(this, Qt::Dialog | Qt::WindowCloseButtonHint);
	dialog->setWindowTitle(tr("Modifier PDC"));

	APsEdit = new QSpinBox(dialog);
	APsEdit->setRange(0, MAX_INT8);
	
	qint8 pos;
	if((pos = posAbility(abilityID)) != -1)
	{
		APsEdit->setValue(gf_data->APs[pos]);
	}
	
	APsMax = new QLabel(QString("/%1").arg(Data::apsTab[abilityID]), dialog);
	abilityCompleted = new QCheckBox(tr("Acquis!"), dialog);
	abilityCompleted->setChecked(getCompleteAbility(abilityID));
	okC = new QPushButton(tr("Fermer"), dialog);
	fullButton = new QPushButton(QString("%1/%2").arg(Data::apsTab[abilityID]).arg(Data::apsTab[abilityID]), dialog);
	
	QGridLayout *grid = new QGridLayout(dialog);
	grid->addWidget(APsEdit, 0, 0);
	grid->addWidget(APsMax, 0, 1);
	grid->addWidget(abilityCompleted, 1, 0, 1, 2);
	grid->addWidget(okC, 2, 0);
	grid->addWidget(fullButton, 2, 1);

	connect(okC, SIGNAL(released()), SLOT(changeAPs()));
	connect(fullButton, SIGNAL(released()), SLOT(fullAPs()));
	
	dialog->exec();
}

void GfEditor::restore_C()
{
	QList<QTreeWidgetItem *> selectedItems = liste2->selectedItems();
	if(selectedItems.isEmpty())	return;

	quint8 abilityID = selectedItems.first()->text(0).toInt();
	delete selectedItems.first();
	qint8 pos = posAbility(abilityID);
	
	QTreeWidgetItem *item;
	
	if(gf_data->APs[pos] < Data::apsTab[abilityID])
	{
		setCompleteAbility(abilityID, false);
		item = addItem(abilityID, 0, pos);
	}
	else
	{
		setCompleteAbility(abilityID, true);
		item = addItem(abilityID, 1);
	}
	
	liste->sortByColumn(0, Qt::AscendingOrder);
	liste->scrollToItem(item);
	liste->setCurrentItem(item);
	liste->header()->setResizeMode(1, QHeaderView::Stretch);
	liste->header()->setResizeMode(2, QHeaderView::ResizeToContents);
	gf_data->forgotten ^= 1 << pos;
}

void GfEditor::enableButtons()
{
	QList<QTreeWidgetItem *> selectedItems = liste->selectedItems();
	if(selectedItems.isEmpty())
	{
		removeC->setEnabled(false);
		learnC->setEnabled(false);
		editC->setEnabled(false);
		return;
	}
	removeC->setEnabled(true);
	learnC->setEnabled(true);
	editC->setEnabled(posAbility(selectedItems.first()->text(0).toInt()) != -1);

	removeC->setText(posAbility(selectedItems.first()->text(0).toInt()) == -1 ? tr("Supprimer") : tr("Oublier"));
}

void GfEditor::enableButtons2()
{
	restoreF->setEnabled(!liste2->selectedItems().isEmpty());
}

void GfEditor::changeExp(double value)
{
	int niv = ((quint32)value/nivSup)+1;
	if(niv>100)	niv = 100;
	if(nivEdit->value() != niv)
	{
		nivEdit->setValue(niv);
	}
}

void GfEditor::changeNiv(int value)
{
	quint32 exp = (value-1)*nivSup;
	if(exp > expEdit->value() || exp + nivSup <= expEdit->value())
	{
		expEdit->setValue(exp);
	}
}

void GfEditor::changeCapacity(int index)
{
	okC->setEnabled(posAbility(index+1) == -1);
}

void GfEditor::addCapacity()
{
	quint8 abilityID = selection->currentIndex()+1;
	if(!getCompleteAbility(abilityID))
	{
		QTreeWidgetItem *item = addItem(abilityID, 1);
		liste->sortByColumn(0, Qt::AscendingOrder);
		
		setCompleteAbility(abilityID, true);
		liste->scrollToItem(item);
		liste->setCurrentItem(item);
		liste->header()->setResizeMode(1, QHeaderView::Stretch);
		liste->header()->setResizeMode(2, QHeaderView::ResizeToContents);
	}
}

void GfEditor::changeAPs()
{
	int value = APsEdit->value();
	QList<QTreeWidgetItem *> selectedItems = liste->selectedItems();
	if(selectedItems.isEmpty())	return;
	
	quint8 abilityID = selectedItems.first()->text(0).toInt();
	qint8 pos;
	if((pos = posAbility(abilityID)) != -1)
	{
		gf_data->APs[pos] = (char)value;
		if(!abilityCompleted->isChecked())
		{
			setCompleteAbility(abilityID, false);
			selectedItems.first()->setText(2, QString("%1/%2").arg(value).arg(Data::apsTab[abilityID]));
		}
		else
		{
			setCompleteAbility(abilityID, true);
			selectedItems.first()->setText(2, tr("Acquis!"));
		}
	}
	dialog->close();
}

void GfEditor::fullAPs()
{
	QList<QTreeWidgetItem *> selectedItems = liste->selectedItems();
	if(selectedItems.isEmpty())	return;
	
	APsEdit->setValue(Data::apsTab[selectedItems.first()->text(0).toInt()]);
}

void GfEditor::changeLearning()
{
	QList<QTreeWidgetItem *> selectedItems = liste->selectedItems();
	if(selectedItems.isEmpty())	return;
	
	QList<QTreeWidgetItem *> findItems = liste->findItems(QString("%1").arg(gf_data->learning,3), Qt::MatchExactly);
	if(!findItems.isEmpty())
	{
		quint8 abilityID = findItems.first()->text(0).toInt();
		findItems.first()->setIcon(1, QIcon(QString(":/images/icons/capacity%1.png").arg(Data::abilityType(abilityID))));
		Qt::GlobalColor color = posAbility(abilityID) != -1 ? Qt::black : Qt::darkGray;
		findItems.first()->setForeground(1, QBrush(QColor(color)));
		findItems.first()->setForeground(2, QBrush(QColor(color)));
	}
	
	if(gf_data->learning != selectedItems.first()->text(0).toInt())
	{
		gf_data->learning = selectedItems.first()->text(0).toInt();
		selectedItems.first()->setIcon(1, QIcon(":/images/icons/learning.png"));
		Qt::GlobalColor color = posAbility(gf_data->learning) != -1 ? Qt::darkGreen : Qt::darkGray;
		selectedItems.first()->setForeground(1, QBrush(QColor(color)));
		selectedItems.first()->setForeground(2, QBrush(QColor(color)));
		liste->scrollToItem(selectedItems.first());
	}
	else
	{
		gf_data->learning = 0;
	}
}

qint8 GfEditor::posAbility(quint8 abilityID)
{
	for(quint8 i=0 ; i<22 ; ++i)
	{
		if(Data::innateAbilities[id][i] == abilityID)
			return i;
	}
	return -1;
}

void GfEditor::setCompleteAbility(quint8 pos, bool value)
{
	gf_data->completeAbilities[pos/8] = value ? ( gf_data->completeAbilities[pos/8] | (1 << (pos%8)) ) : ( gf_data->completeAbilities[pos/8] & ~(1 << (pos%8)) );
}

bool GfEditor::getCompleteAbility(quint8 pos)
{
	return ((quint8)gf_data->completeAbilities[pos/8] >> pos%8) & 1;
}
