/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "ItemEditor.h"

ItemEditor::ItemEditor(QWidget *parent) : QWidget(parent)
{
	// buildWidget();
	builded = false;
}

void ItemEditor::load(MAIN *data)
{
	if(!builded)
		buildWidget();
	this->data = data;
	fillPage();
}

void ItemEditor::buildWidget()
{
	builded = true;
	tabWidget = new QTabWidget(this);
	tabWidget->addTab(buildPage1(), tr("Objets"));
	tabWidget->addTab(buildPage2(), tr("Ordre en combat"));

	QVBoxLayout *layout = new QVBoxLayout(this);
	layout->addWidget(tabWidget);
	layout->setContentsMargins(QMargins());
}

QWidget *ItemEditor::buildPage1()
{
	QWidget *ret = new QWidget(this);

	QFont font;
	font.setPointSize(10);
	itemE_model = new QStandardItemModel(ret);
	itemE_view = new QTreeView(ret);
	itemE_view->setModel(itemE_model);
	itemE_view->setEditTriggers(QAbstractItemView::DoubleClicked);
	itemE_view->setFont(font);
	itemE_view->setItemDelegate(new SpinBoxDelegate(ret));
	itemE_view->setIndentation(0);
	itemE_view->setItemsExpandable(false);
	itemE_view->setUniformRowHeights(true);
	itemE_view->header()->setStretchLastSection(false);
	itemE_view->setAllColumnsShowFocus(true);

	QWidget *buttonsW = new QWidget(ret);

	QPushButton *allItems = new QPushButton(tr("Tous les objets"), buttonsW);
	QPushButton *type_sort = new QPushButton(tr("Type"), buttonsW);
	QPushButton *alpha_sort = new QPushButton(tr("Ordre alphabtique"), buttonsW);

	QHBoxLayout *buttons = new QHBoxLayout(buttonsW);
	buttons->addWidget(allItems);
	buttons->addStretch();
	buttons->addWidget(new QLabel(tr("Trier par :"), buttonsW));
	buttons->addWidget(type_sort);
	buttons->addWidget(alpha_sort);
	buttons->setContentsMargins(QMargins());
	
	QVBoxLayout *layout = new QVBoxLayout(ret);
	layout->addWidget(itemE_view);
	layout->addWidget(buttonsW);

	connect(allItems, SIGNAL(released()), SLOT(allItems()));
	connect(type_sort, SIGNAL(released()), SLOT(sortByType()));
	connect(alpha_sort, SIGNAL(released()), SLOT(sortByAlpha()));

	return ret;
}

QWidget *ItemEditor::buildPage2()
{
	QWidget *ret = new QWidget(this);

	QFont font;
	font.setPointSize(10);

	battle_itemE_list = new QListWidget(ret);
	battle_itemE_list->setFont(font);
	battle_itemE_list->setDragDropMode(QAbstractItemView::InternalMove);

	QVBoxLayout *layout = new QVBoxLayout(ret);
	layout->addWidget(battle_itemE_list);

	return ret;
}

void ItemEditor::fillPage()
{
	quint8 itemID;
	QStandardItem *standardItem;
	QListWidgetItem *item;
	QList<quint8> items;
	QMultiMap<int, int> battle_order;
	
	itemE_model->clear();
	
	itemE_model->setHorizontalHeaderLabels(QStringList() << tr("Nom") << tr("Qt"));
	
	for(int i=0 ; i<198 ; ++i)
	{
		itemID = data->objets.items[i] & 0xFF;

		if(itemID != 0)		items.append(itemID);
		
		QList<QStandardItem *> items;
		standardItem = new QStandardItem(Data::items.value(itemID, QString::number(itemID)));
		standardItem->setData(2, Qt::UserRole);
		standardItem->setData(itemID, Qt::UserRole+1);
		standardItem->setIcon(itemID == 0 ? QIcon() : QIcon(QString(":/images/icons/objet%1.png").arg(Data::itemType(itemID))));
		items << standardItem;
		
		standardItem = new QStandardItem(QString::number(data->objets.items[i] >> 8));
		standardItem->setData(0, Qt::UserRole);
		items << standardItem;
		itemE_model->appendRow(items);
	}
	
	battle_itemE_list->clear();
	itemE_view->header()->setResizeMode(0, QHeaderView::Stretch);
	itemE_view->header()->setResizeMode(1, QHeaderView::ResizeToContents);

	for(itemID=0 ; itemID<32 ; ++itemID)
	{
		// (pos, id)
		battle_order.insert(data->objets.battle_order[itemID], itemID+1);
	}

	foreach(int itemID2, battle_order)
	{
		item = new QListWidgetItem(QIcon(QString(":/images/icons/objet%1.png").arg(Data::itemType(itemID2))),
								   itemID2 < 33 ? Data::items.at(itemID2) : QString::number(itemID2));
		if(!items.contains(itemID2))
			item->setForeground(QBrush(QColor(0xCC,0xCC,0xCC)));
		item->setData(Qt::UserRole, QVariant(itemID2));
		item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled);

		battle_itemE_list->addItem(item);
	}
}

void ItemEditor::savePage()
{
	int itemID;

	for(int i=0 ; i<198 ; ++i)
	{
		data->objets.items[i] = (quint8)itemE_model->item(i, 0)->data().toInt();
		data->objets.items[i] |= (quint8)itemE_model->item(i, 1)->text().toUInt() << 8;
	}

	for(int pos=0 ; pos<32 ; ++pos)
	{
		itemID = battle_itemE_list->item(pos)->data(Qt::UserRole).toInt()-1;
		data->objets.battle_order[itemID] = pos;
	}
}

void ItemEditor::allItems()
{
	QStandardItem *item;
	for(int itemID=1 ; itemID<199 ; ++itemID)
	{
		item = itemE_model->item(itemID-1, 0);
		item->setData(itemID);
		item->setText(Data::items.at(itemID));
		item->setIcon(QIcon(QString(":/images/icons/objet%1.png").arg(Data::itemType(itemID))));
		itemE_model->item(itemID-1, 1)->setText(QString::number(100));
	}
}

void ItemEditor::sortByType()
{
	QMultiMap<int, quint8> items;
	int itemID, i;

	for(i=0 ; i<198 ; ++i)
	{
		itemID = itemE_model->item(i, 0)->data().toInt();
		if(itemID == 0)		itemID = 256;
		items.insert(itemID, itemE_model->item(i, 1)->text().toUInt());
	}

	QMultiMap<int, quint8>::const_iterator it = items.constBegin();

	i = 0;
	while(it != items.constEnd())
	{
		itemID = it.key();
		if(itemID == 256)		itemID = 0;
		data->objets.items[i++] = itemID | (it.value() << 8);
		++it;
	}

	fillPage();
}

void ItemEditor::sortByAlpha()
{
	QMultiMap<QString, quint16> items;
	QStandardItem *item;
	int i;
	QString text;

	for(i=0 ; i<198 ; ++i)
	{
		item = itemE_model->item(i, 0);
		text = item->text();
		if(text.isEmpty())	text = "zzzzzzzzzzzzzzzzzz";
		items.insert(text, item->data().toInt() | (itemE_model->item(i, 1)->text().toUInt() << 8));
	}

	i = 0;
	foreach(quint16 value, items)
	{
		data->objets.items[i++] = value;
	}

	fillPage();
}
