/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "PersoEditor.h"

PersoEditor::PersoEditor(QWidget *parent) : QWidget(parent)
{
	// buildWidget();
	builded = false;
	this->id = 255;
}

void PersoEditor::load(MAIN *data, HEADER *descData, bool jp)
{
	if(!builded)
		buildWidget();
	this->data = data;
	this->id = 255;
	this->descData = descData;
	this->jp = jp;
	fillPage();
}

void PersoEditor::updateMode()
{
    inconnu_E->setVisible(Config::mode());
	inconnu1_E->setVisible(Config::mode());
	inconnu1_label_E->setVisible(Config::mode());
}

void PersoEditor::buildWidget()
{
	builded = true;
	QVBoxLayout *layout = new QVBoxLayout(this);
	layout->setContentsMargins(QMargins());
	persoListe = new QListWidget(this);
	persoListe->setViewMode(QListView::IconMode);
	persoListe->setFlow(QListView::LeftToRight);
	persoListe->setIconSize(QSize(32,48));
	persoListe->setFixedHeight(57);
	persoListe->setWrapping(false);
	persoListe->setMovement(QListView::Static);
	
	tabWidget = new QTabWidget(this);
	tabWidget->addTab(buildPage1(), tr("tat"));
	tabWidget->addTab(buildPage2(), tr("G-Forces"));
	tabWidget->addTab(buildPage3(), tr("Magies"));
	tabWidget->addTab(buildPage4(), tr("Capacits"));
	tabWidget->addTab(buildPage5(), tr("Associations"));
	lbWidget = new QWidget(this);
	tabWidget->addTab(lbWidget, tr("Limit Break"));
	
	layout->addWidget(persoListe);
	layout->addWidget(tabWidget);
//	layout->addStretch();
	
	QListWidgetItem *item;
	for(quint8 i=0 ; i<8 ; ++i)
	{
		item = new QListWidgetItem(persoListe);
		item->setIcon(QIcon(QString(":/images/icons/perso%1.png").arg(i)));
		item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
	}

	persoListe->setCurrentRow(0);
	
	connect(persoListe, SIGNAL(currentRowChanged(int)), SLOT(fillPage(int)));
}

QWidget *PersoEditor::buildPage1()
{
	QWidget *ret = new QWidget(this);
	
	existsE = new QCheckBox(ret);
	nameE = new QLineEdit(ret);
//	nameE->setMaxLength(11);
	current_HPsE = new QSpinBox(ret);
	current_HPsE->setRange(0, MAX_INT16);
	
	QHBoxLayout *mainEdit_l = new QHBoxLayout;
	mainEdit_l->addWidget(new QLabel(tr("Disponible"), ret));
	mainEdit_l->addWidget(existsE);
	mainEdit_l->addWidget(new QLabel(tr("Nom :"), ret));
	mainEdit_l->addWidget(nameE);
	mainEdit_l->addWidget(new QLabel(tr("HP actuels :"), ret));
	mainEdit_l->addWidget(current_HPsE);
	
	expE = new QDoubleSpinBox(ret);
	expE->setDecimals(0);
	expE->setRange(0, MAX_INT32);
	connect(expE, SIGNAL(valueChanged(double)), SLOT(exp_S(double)));
	nivE = new QSpinBox(ret);
	nivE->setRange(1, 100);
	connect(nivE, SIGNAL(valueChanged(int)), SLOT(niv_S(int)));
	
	QHBoxLayout *mainEdit2_l = new QHBoxLayout;
	mainEdit2_l->addWidget(new QLabel(tr("Niv :"), ret));
	mainEdit2_l->addWidget(nivE);
	mainEdit2_l->addWidget(new QLabel(tr("EXP :"), ret));
	mainEdit2_l->addWidget(expE);
	
	kills_E = new QSpinBox(ret);
	kills_E->setRange(0, MAX_INT16);
	kos_E = new QSpinBox(ret);
	kos_E->setRange(0, MAX_INT16);
	
	QHBoxLayout *statsEdit_l = new QHBoxLayout;
	statsEdit_l->addWidget(new QLabel(tr("Tus :"), ret));
	statsEdit_l->addWidget(kills_E);
	statsEdit_l->addWidget(new QLabel(tr("Mort :"), ret));
	statsEdit_l->addWidget(kos_E);
	
	id_E = new QComboBox(ret);
	for(quint8 i=0 ; i<11 ; ++i)
		id_E->addItem(QIcon(QString(":/images/icons/perso%1.png").arg(i)), Data::names.at(i));
	for(quint16 i=11 ; i<256 ; ++i)
		id_E->addItem(QString::number(i));

	weapon_E = new QComboBox(ret);
	for(quint8 i=0 ; i<28 ; ++i)
		weapon_E->addItem(QIcon(QString(":/images/icons/weapon%1.png").arg(i,2,10,QChar('0'))), Data::weapons.at(i));
	for(quint8 i=28 ; i<33 ; ++i)
		weapon_E->addItem(Data::weapons.at(i));
	for(quint16 i=33 ; i<256 ; ++i)
		weapon_E->addItem(QString::number(i));
	
	QHBoxLayout *statsEdit2_l = new QHBoxLayout;
	statsEdit2_l->addWidget(new QLabel(tr("Personnage :"), ret));
	statsEdit2_l->addWidget(id_E);
	statsEdit2_l->addWidget(new QLabel(tr("Arme :"), ret));
	statsEdit2_l->addWidget(weapon_E);

	status_E = new QGroupBox(tr("Statut"), ret);
	QGridLayout *statusEdit_l = new QGridLayout(status_E);

	QStringList status;
	status << tr("Mort") << tr("Poison") << tr("Fossile") << tr("Darkness") << tr("Aphasie") << tr("Trouble") << tr("Zombie");
	QCheckBox *status_checkBox;
	for(int i=0 ; i<7 ; ++i)
	{
		status_checkBox = new QCheckBox(status.at(i), status_E);
		status_checkBox->setIcon(QIcon(QString(":/images/icons/mtl0%1.png").arg(i)));
		status_list_E.append(status_checkBox);
		statusEdit_l->addWidget(status_checkBox, i/4, i%4);
	}
	status_checkBox = new QCheckBox("???", status_E);
	status_list_E.append(status_checkBox);
	statusEdit_l->addWidget(status_checkBox, 1, 3);
	
	inconnu_E = new QGroupBox(tr("Inconnu"), ret);
	inconnu2_E = new QSpinBox(inconnu_E);
	inconnu2_E->setRange(0, MAX_INT8);
	inconnu3_E = new QSpinBox(inconnu_E);
	inconnu3_E->setRange(0, MAX_INT8);
	inconnu4_E = new QSpinBox(inconnu_E);
	inconnu4_E->setRange(0, MAX_INT8);
	inconnu5_E = new QSpinBox(inconnu_E);
	inconnu5_E->setRange(0, MAX_INT8);
	inconnu6_E = new QSpinBox(inconnu_E);
	inconnu6_E->setRange(0, 31);
	
	QHBoxLayout *inconnuEdit_l = new QHBoxLayout(inconnu_E);
	inconnuEdit_l->addWidget(new QLabel(tr("1 :"), inconnu_E));
	inconnuEdit_l->addWidget(inconnu2_E);
	inconnuEdit_l->addWidget(new QLabel(tr("2 :"), inconnu_E));
	inconnuEdit_l->addWidget(inconnu3_E);
	inconnuEdit_l->addWidget(new QLabel(tr("3 :"), inconnu_E));
	inconnuEdit_l->addWidget(inconnu4_E);
	inconnuEdit_l->addWidget(new QLabel(tr("4 :"), inconnu_E));
	inconnuEdit_l->addWidget(inconnu5_E);
	inconnuEdit_l->addWidget(new QLabel(tr("5 :"), inconnu_E));
	inconnuEdit_l->addWidget(inconnu6_E);
	
	bonus_E = new QGroupBox(tr("Bonus"), ret);
	HPs_E = new QSpinBox(bonus_E);
	HPs_E->setRange(0, MAX_INT16);
	vgr_E = new QSpinBox(bonus_E);
	vgr_E->setRange(0, MAX_INT8);
	dfs_E = new QSpinBox(bonus_E);
	dfs_E->setRange(0, MAX_INT8);
	mgi_E = new QSpinBox(bonus_E);
	mgi_E->setRange(0, MAX_INT8);
	psy_E = new QSpinBox(bonus_E);
	psy_E->setRange(0, MAX_INT8);
	vts_E = new QSpinBox(bonus_E);
	vts_E->setRange(0, MAX_INT8);
	chc_E = new QSpinBox(bonus_E);
	chc_E->setRange(0, MAX_INT8);
	
	QGridLayout *bonusEdit_l = new QGridLayout(bonus_E);
	bonusEdit_l->addWidget(new QLabel(tr("Bonus HP :"), bonus_E), 0, 0);
	bonusEdit_l->addWidget(HPs_E, 0, 1);
	bonusEdit_l->addWidget(new QLabel(tr("Bonus vgr :"), bonus_E), 0, 2);
	bonusEdit_l->addWidget(vgr_E, 0, 3);
	bonusEdit_l->addWidget(new QLabel(tr("Bonus dfs :"), bonus_E), 0, 4);
	bonusEdit_l->addWidget(dfs_E, 0, 5);
	bonusEdit_l->addWidget(new QLabel(tr("Bonus mgi :"), bonus_E), 0, 6);
	bonusEdit_l->addWidget(mgi_E, 0, 7);
	bonusEdit_l->addWidget(new QLabel(tr("Bonus psy :"), bonus_E), 1, 0);
	bonusEdit_l->addWidget(psy_E, 1, 1);
	bonusEdit_l->addWidget(new QLabel(tr("Bonus vts :"), bonus_E), 1, 2);
	bonusEdit_l->addWidget(vts_E, 1, 3);
	bonusEdit_l->addWidget(new QLabel(tr("Bonus chc :"), bonus_E), 1, 4);
	bonusEdit_l->addWidget(chc_E, 1, 5);

	alternativeE = new QSpinBox(ret);
	alternativeE->setRange(0, MAX_INT8);
	lock1E = new QCheckBox(tr("Vrouill 1 (menu 'remplacer')"), ret);
	lock2E = new QCheckBox(tr("Vrouill 2 (menu 'remplacer')"), ret);

	QHBoxLayout *alternativeL = new QHBoxLayout();
	alternativeL->addWidget(new QLabel(tr("Costume alternatif (Seed, galbadien) :"), inconnu_E));
	alternativeL->addWidget(alternativeE);
	alternativeL->addWidget(lock1E);
	alternativeL->addWidget(lock2E);
	
	QGridLayout *grid = new QGridLayout(ret);
	grid->addLayout(mainEdit_l, 0, 0);
	grid->addLayout(mainEdit2_l, 1, 0);
	grid->addLayout(statsEdit_l, 2, 0);
	grid->addLayout(statsEdit2_l, 3, 0);
	grid->addWidget(status_E, 4, 0);
	grid->addWidget(bonus_E, 5, 0);
	grid->addWidget(inconnu_E, 6, 0);
	grid->addLayout(alternativeL, 7, 0);
	grid->setRowStretch(8, 1);
	
	return ret;
}

QWidget *PersoEditor::buildPage2()
{
	QWidget *ret = new QWidget(this);
	
	for(int i=0 ; i<16 ; ++i)
	{
		poss_E << new QCheckBox(Data::gfnames.at(i), ret);
		poss_E.at(i)->setIcon(QIcon(QString(":/images/icons/gf%1.png").arg(i)));
		comp_E << new QSpinBox(ret);
		comp_E.at(i)->setRange(-59535, 6000);
		compLabel_E << new QLabel(ret);
		compLabel_E.at(i)->setTextFormat(Qt::PlainText);
		compLabel_E.at(i)->setTextInteractionFlags(Qt::NoTextInteraction);
		connect(comp_E.at(i), SIGNAL(valueChanged(int)), SLOT(updateCompLabels()));
	}
	
	QGridLayout *grid = new QGridLayout(ret);
	for(int i=0 ; i<8 ; ++i)
	{
		for(int j=0 ; j<2 ; j++)
		{
			grid->addWidget(poss_E.at(i), i, 0);
			grid->addWidget(comp_E.at(i), i, 1, Qt::AlignRight);
			grid->addWidget(compLabel_E.at(i), i, 2, Qt::AlignLeft);

			grid->addWidget(poss_E.at(i+8), i, 3);
			grid->addWidget(comp_E.at(i+8), i, 4, Qt::AlignRight);
			grid->addWidget(compLabel_E.at(i+8), i, 5, Qt::AlignLeft);
		}
	}
	grid->setRowStretch(8, 1);
	
	return ret;
}

QWidget *PersoEditor::buildPage3()
{
	QWidget *ret = new QWidget(this);

	QFont font;
	font.setPointSize(10);
	
	magie_E_model = new QStandardItemModel(ret);
	magie_E_view = new QTreeView(ret);
	magie_E_view->setModel(magie_E_model);
	magie_E_view->setEditTriggers(QAbstractItemView::DoubleClicked);
	magie_E_view->setFont(font);
	magie_E_view->setItemDelegate(new SpinBoxDelegate(ret));
	magie_E_view->setIndentation(0);
	magie_E_view->setItemsExpandable(false);
	magie_E_view->header()->setStretchLastSection(false);
	magie_E_view->setUniformRowHeights(true);
	magie_E_view->setAllColumnsShowFocus(true);
	
	QVBoxLayout *layout = new QVBoxLayout(ret);
	layout->addWidget(magie_E_view);
	
	return ret;
}

QWidget *PersoEditor::buildPage4()
{
	QWidget *ret = new QWidget(this);
	
	QGroupBox *commande_E = new QGroupBox(tr("Commandes"), ret);
	commande1_E = new QComboBox(commande_E);
	commande1_E->addItem("-");
	Data::fillAbilities(commande1_E);
	commande2_E = new QComboBox(commande_E);
	commande2_E->addItem("-");
	Data::fillAbilities(commande2_E);
	commande3_E = new QComboBox(commande_E);
	commande3_E->addItem("-");
	Data::fillAbilities(commande3_E);
	inconnu1_label_E = new QLabel(tr("Inconnu (commande 4) :"), commande_E);
	inconnu1_E = new QSpinBox(commande_E);
	inconnu1_E->setRange(0, MAX_INT8);
	
	QGridLayout *commande_L = new QGridLayout(commande_E);
	commande_L->addWidget(new QLabel(tr("Commande 1 :"), commande_E), 0, 0);
	commande_L->addWidget(commande1_E, 0, 1);
	commande_L->addWidget(new QLabel(tr("Commande 2 :"), commande_E), 1, 0);
	commande_L->addWidget(commande2_E, 1, 1);
	commande_L->addWidget(new QLabel(tr("Commande 3 :"), commande_E), 2, 0);
	commande_L->addWidget(commande3_E, 2, 1);
	commande_L->addWidget(inconnu1_label_E, 3, 0);
	commande_L->addWidget(inconnu1_E, 3, 1);
	commande_L->setRowStretch(4, 1);
	
	QGroupBox *ability_E = new QGroupBox(tr("Capacits"), ret);
	ability1_E = new QComboBox(ability_E);
	ability1_E->addItem("-");
	Data::fillAbilities(ability1_E);
	ability2_E = new QComboBox(ability_E);
	ability2_E->addItem("-");
	Data::fillAbilities(ability2_E);
	ability3_E = new QComboBox(ability_E);
	ability3_E->addItem("-");
	Data::fillAbilities(ability3_E);
	ability4_E = new QComboBox(ability_E);
	ability4_E->addItem("-");
	Data::fillAbilities(ability4_E);
	
	QGridLayout *ability_L = new QGridLayout(ability_E);
	ability_L->addWidget(new QLabel(tr("Capacit 1 :"), ability_E), 0, 0);
	ability_L->addWidget(ability1_E, 0, 1);
	ability_L->addWidget(new QLabel(tr("Capacit 2 :"), ability_E), 1, 0);
	ability_L->addWidget(ability2_E, 1, 1);
	ability_L->addWidget(new QLabel(tr("Capacit 3 :"), ability_E), 2, 0);
	ability_L->addWidget(ability3_E, 2, 1);
	ability_L->addWidget(new QLabel(tr("Capacit 4 :"), ability_E), 3, 0);
	ability_L->addWidget(ability4_E, 3, 1);
	ability_L->setRowStretch(4, 1);
	
	QHBoxLayout *layout = new QHBoxLayout(ret);
	layout->addWidget(commande_E);
	layout->addWidget(ability_E);
	
	return ret;
}

QWidget *PersoEditor::buildPage5()
{
	QWidget *ret = new QWidget(this);
	QGroupBox *apt_E = new QGroupBox(tr("Aptitudes"), ret);
	for(int i=0 ; i<9 ; ++i)
	{
		aptitude_E << new QComboBox(apt_E);
		aptitude_E[i]->addItems(Data::magic);
	}
	QGridLayout *apt_L = new QGridLayout(apt_E);
	apt_L->addWidget(new QLabel(tr("HP :"), apt_E), 0, 0);
	apt_L->addWidget(aptitude_E[0], 0, 1);
	apt_L->addWidget(new QLabel(tr("Vigueur :"), apt_E), 1, 0);
	apt_L->addWidget(aptitude_E[1], 1, 1);
	apt_L->addWidget(new QLabel(tr("Dfense :"), apt_E), 2, 0);
	apt_L->addWidget(aptitude_E[2], 2, 1);
	apt_L->addWidget(new QLabel(tr("Magie :"), apt_E), 3, 0);
	apt_L->addWidget(aptitude_E[3], 3, 1);
	apt_L->addWidget(new QLabel(tr("Psychisme :"), apt_E), 4, 0);
	apt_L->addWidget(aptitude_E[4], 4, 1);
	
	apt_L->addWidget(new QLabel(tr("Vitesse :"), apt_E), 1, 2);
	apt_L->addWidget(aptitude_E[5], 1, 3);
	apt_L->addWidget(new QLabel(tr("Esquive :"), apt_E), 2, 2);
	apt_L->addWidget(aptitude_E[6], 2, 3);
	apt_L->addWidget(new QLabel(tr("Protection :"), apt_E), 3, 2);
	apt_L->addWidget(aptitude_E[7], 3, 3);
	apt_L->addWidget(new QLabel(tr("Chance :"), apt_E), 4, 2);
	apt_L->addWidget(aptitude_E[8], 4, 3);
	
	QGroupBox *ele_E = new QGroupBox(tr("lmental"), ret);
	for(int i=0 ; i<5 ; ++i)
	{
		element_E << new QComboBox(ele_E);
		element_E[i]->addItems(Data::magic);
	}
	QGridLayout *ele_L = new QGridLayout(ele_E);
	ele_L->addWidget(new QLabel(tr("Attaque :"), ele_E), 0, 0);
	ele_L->addWidget(element_E[0], 0, 1);
	ele_L->addWidget(new QLabel(tr("Dfense 1 :"), ele_E), 1, 0);
	ele_L->addWidget(element_E[1], 1, 1);
	ele_L->addWidget(new QLabel(tr("Dfense 2 :"), ele_E), 2, 0);
	ele_L->addWidget(element_E[2], 2, 1);
	ele_L->addWidget(new QLabel(tr("Dfense 3 :"), ele_E), 3, 0);
	ele_L->addWidget(element_E[3], 3, 1);
	ele_L->addWidget(new QLabel(tr("Dfense 4 :"), ele_E), 4, 0);
	ele_L->addWidget(element_E[4], 4, 1);
	
	QGroupBox *mtl_E = new QGroupBox(tr("Mental"), ret);
	for(int i=0 ; i<5 ; ++i)
	{
		mental_E << new QComboBox(mtl_E);
		mental_E[i]->addItems(Data::magic);
	}
	QGridLayout *mtl_L = new QGridLayout(mtl_E);
	mtl_L->addWidget(new QLabel(tr("Attaque :"), mtl_E), 0, 0);
	mtl_L->addWidget(mental_E[0], 0, 1);
	mtl_L->addWidget(new QLabel(tr("Dfense 1 :"), mtl_E), 1, 0);
	mtl_L->addWidget(mental_E[1], 1, 1);
	mtl_L->addWidget(new QLabel(tr("Dfense 2 :"), mtl_E), 2, 0);
	mtl_L->addWidget(mental_E[2], 2, 1);
	mtl_L->addWidget(new QLabel(tr("Dfense 3 :"), mtl_E), 3, 0);
	mtl_L->addWidget(mental_E[3], 3, 1);
	mtl_L->addWidget(new QLabel(tr("Dfense 4 :"), mtl_E), 4, 0);
	mtl_L->addWidget(mental_E[4], 4, 1);
	
	QGridLayout *grid = new QGridLayout(ret);
	grid->addWidget(apt_E, 0, 0, 1, 2);
	grid->addWidget(ele_E, 1, 0);
	grid->addWidget(mtl_E, 1, 1);
	grid->setRowStretch(2, 1);
	
	return ret;
}

void PersoEditor::buildPage6()
{
	foreach(QObject *child, lbWidget->children())	delete child;
	//Cette fonction varie en fonction du personnage, elle est appele lors du remplissage et pas lors de l'initialisation
	QGridLayout *grid = new QGridLayout(lbWidget);
	
	int cur=0;
	LB_E.clear();
	linoaLB_E.clear();
	tabWidget->setTabEnabled(5, true);
	QLabel *indicatorlabel;
	QLabel *angeloPix;
	
	switch(id)
	{
	case 0://Squall
		LB_E << new QCheckBox(tr("Gunblade auto"), lbWidget);
		LB_E.first()->setChecked(data->stats.renzokuken_auto & 1);
		LB_E << new QCheckBox(tr("Indicateur renzokuken"), lbWidget);
		LB_E.at(1)->setChecked(data->stats.renzokuken_indicator < 128);
		LB_E.at(1)->setEnabled(data->stats.renzokuken_auto == 0);
		LBindicator_E = new QSlider(Qt::Horizontal, lbWidget);
		LBindicator_E->setMaximum(127);
		LBindicator_E->setValue(data->stats.renzokuken_indicator < 128 ? data->stats.renzokuken_indicator : 0);
		LBindicator_E->setEnabled(data->stats.renzokuken_auto == 0 || data->stats.renzokuken_indicator < 128);
		connect(LB_E.first(), SIGNAL(stateChanged(int)), SLOT(LBautoChange(int)));
		connect(LB_E.at(1), SIGNAL(stateChanged(int)), SLOT(LBindicatorChange(int)));
		indicatorlabel = new QLabel(QString::number(data->stats.renzokuken_indicator < 128 ? data->stats.renzokuken_indicator : 0), lbWidget);
		connect(LBindicator_E, SIGNAL(valueChanged(int)), indicatorlabel, SLOT(setNum(int)));
		grid->addWidget(LB_E.first(), 0, 0);
		grid->addWidget(LB_E.at(1), 1, 0);
		grid->addWidget(LBindicator_E, 1, 1);
		grid->addWidget(indicatorlabel, 1, 2);
		grid->setRowStretch(2, 1);
		grid->setColumnStretch(3, 1);
		return;
	case 1://Zell
		for(int i=0 ; i<2 ; ++i)
		{
			for(int j=0 ; j<5 ; ++j)
			{
				LB_E << new QCheckBox(Data::zellLB.at(cur), lbWidget);
				grid->addWidget(LB_E.at(cur), i, j);
				LB_E.at(cur)->setChecked((data->limitb.zell >> cur) & 1);
				cur++;
			}
		}
		grid->setRowStretch(2, 1);
		return;
	case 2://Irvine
		for(int i=0 ; i<2 ; ++i)
		{
			for(int j=0 ; j<4 ; ++j)
			{
				LB_E << new QCheckBox(Data::irvineLB.at(cur), lbWidget);
				grid->addWidget(LB_E.at(cur), i, j);
				LB_E.at(cur)->setChecked((data->limitb.irvine >> cur) & 1);
				cur++;
			}
		}
		grid->setRowStretch(2, 1);
		return;
	case 3://Quistis
		for(int i=0 ; i<4 ; ++i)
		{
			for(int j=0 ; j<4 ; ++j)
			{
				LB_E << new QCheckBox(Data::quistisLB.at(cur), lbWidget);
				grid->addWidget(LB_E.at(cur), i, j);
				LB_E.at(cur)->setChecked((data->limitb.quistis >> cur) & 1);
				cur++;
			}
		}
		grid->setRowStretch(4, 1);
		return;
	case 4://Linoa
		angel_E = new QLineEdit(FF8Text::toString(descData->angelo, jp), lbWidget);
//		angel_E->setMaxLength(11);
		angel_disabledE = new QCheckBox(tr("Angel dsactiv"), lbWidget);
		angel_disabledE->setChecked((data->stats.dream >> 4) & 1);
		a_wing_enabledE = new QCheckBox(tr("Canonisation activ"), lbWidget);
		a_wing_enabledE->setChecked((data->stats.dream >> 5) & 1);
		angeloPix = new QLabel(lbWidget);
		angeloPix->setPixmap(QPixmap(":/images/icons/perso15.png"));

		grid->addWidget(new QLabel(Data::names.at(15)+tr(" :"), lbWidget), 0, 0);
		grid->addWidget(angel_E, 0, 1, 1, 6);
		grid->addWidget(new QLabel(tr("<b>Case coche :</b> limit break appris<br/><b>Case partiellement coche :</b> limit break connu<br/><b>Valeur :</b> nombre de points restants pour apprendre le limit break"), lbWidget), 1, 0, 1, 7);
		grid->addWidget(angeloPix, 0, 7, 3, 1, Qt::AlignRight);

		for(int i=2 ; i<4 ; ++i)
		{
			for(int j=0 ; j<4 ; ++j)
			{
				LB_E << new QCheckBox(Data::rinoaLB.at(cur), lbWidget);
				LB_E.at(cur)->setTristate(true);
				linoaLB_E << new QSpinBox(lbWidget);
				linoaLB_E.at(cur)->setRange(0, MAX_INT8);

				grid->addWidget(LB_E.at(cur), i, j*2);
				grid->addWidget(linoaLB_E.at(cur), i, j*2+1);

				LB_E.at(cur)->setCheckState(((data->limitb.angel_known >> cur) & 1) ? Qt::PartiallyChecked : Qt::Unchecked);
				if((data->limitb.angel_completed >> cur) & 1)	LB_E.at(cur)->setCheckState(Qt::Checked);
				linoaLB_E.at(cur)->setValue(data->limitb.angel_pts[cur]);

				++cur;
			}
		}
		grid->addWidget(angel_disabledE, 4, 0, 1, 8);
		grid->addWidget(a_wing_enabledE, 5, 0, 1, 8);
		grid->setRowStretch(6, 1);
		return;
	case 5://Selphie
		for(int i=0 ; i<2 ; ++i)
		{
			for(int j=0 ; j<3 ; ++j)
			{
				LB_E << new QCheckBox(Data::magic.at(cur+51), lbWidget);
				grid->addWidget(LB_E.at(cur), i, j);
				LB_E.at(cur)->setChecked((data->limitb.selphie >> cur) & 1);
				cur++;
			}
		}
		grid->setRowStretch(2, 1);
		return;
	default:
		tabWidget->setTabEnabled(5, false);
		return;
	}
}

void PersoEditor::fillPage(int)
{
	savePage();

	if(persoListe->currentRow()<0)	return;

	this->id = persoListe->currentRow();
	this->perso_data = &data->persos[id];

	QString persoName;
	if(id == 0)			persoName = FF8Text::toString(descData->squall, jp);
	else if(id == 4)	persoName = FF8Text::toString(descData->linoa, jp);
	else				persoName = Data::names.at(id);

	/* PAGE 1 */

	existsE->setChecked(perso_data->exists & 1);
	nameE->setText(persoName);
	nameE->setDisabled(id != 0 && id != 4);

	current_HPsE->setValue(perso_data->current_HPs);

	HPs_E->setValue(perso_data->HPs);
	expE->setValue(perso_data->exp);
	vgr_E->setValue(perso_data->VGR);
	dfs_E->setValue(perso_data->DFS);
	mgi_E->setValue(perso_data->MGI);
	psy_E->setValue(perso_data->PSY);
	vts_E->setValue(perso_data->VTS);
	chc_E->setValue(perso_data->CHC);
	kills_E->setValue(perso_data->kills);
	kos_E->setValue(perso_data->KOs);

	inconnu1_E->setValue(perso_data->u1);
	inconnu2_E->setValue(perso_data->u2);
	inconnu3_E->setValue(perso_data->u3);
	inconnu4_E->setValue(perso_data->u4);
	inconnu5_E->setValue(perso_data->u5);
	inconnu6_E->setValue(perso_data->exists >> 3);
	alternativeE->setValue(perso_data->alternative_model);
	lock1E->setChecked((perso_data->exists >> 1) & 1);
	lock2E->setChecked((perso_data->exists >> 2) & 1);

	id_E->setCurrentIndex(perso_data->ID);
	weapon_E->setCurrentIndex(perso_data->weaponID);

	for(int i=0 ; i<8 ; ++i)
		status_list_E.at(i)->setChecked((perso_data->status >> i) & 1);

	/* PAGE 2 */
	
	for(int i=0 ; i<16 ; ++i)
	{
		poss_E[i]->setChecked((perso_data->gfs >> i) & 1);
		comp_E[i]->setValue(6000-perso_data->compatibility[i]);
	}

	/* PAGE 3 */

	quint8 magID;
	QStandardItem *standardItem;

	magie_E_model->clear();
	magie_E_model->setHorizontalHeaderLabels(QStringList() << tr("Nom") << tr("Qt"));
	
	for(int i=0 ; i<32 ; ++i)
	{
		magID = perso_data->magies[i] & 0xFF;

		QList<QStandardItem *> items;
		standardItem = new QStandardItem(magID < 57 ? Data::magic.at(magID) : QString::number(magID));
		standardItem->setData(1, Qt::UserRole);
		standardItem->setData(magID, Qt::UserRole+1);
		items << standardItem;

		standardItem = new QStandardItem(QString::number(perso_data->magies[i] >> 8));
		standardItem->setData(0, Qt::UserRole);
		items << standardItem;

		magie_E_model->appendRow(items);
	}
	magie_E_view->header()->setResizeMode(0, QHeaderView::Stretch);
	magie_E_view->header()->setResizeMode(1, QHeaderView::ResizeToContents);

	/* PAGE 4 */

	commande1_E->setCurrentIndex(perso_data->commands[0]);
	commande2_E->setCurrentIndex(perso_data->commands[1]);
	commande3_E->setCurrentIndex(perso_data->commands[2]);
	ability1_E->setCurrentIndex(perso_data->abilities[0]);
	ability2_E->setCurrentIndex(perso_data->abilities[1]);
	ability3_E->setCurrentIndex(perso_data->abilities[2]);
	ability4_E->setCurrentIndex(perso_data->abilities[3]);

	/* PAGE 5 */

	aptitude_E[0]->setCurrentIndex(perso_data->j_HP);
	aptitude_E[1]->setCurrentIndex(perso_data->j_VGR);
	aptitude_E[2]->setCurrentIndex(perso_data->j_DFS);
	aptitude_E[3]->setCurrentIndex(perso_data->j_MGI);
	aptitude_E[4]->setCurrentIndex(perso_data->j_PSY);
	aptitude_E[5]->setCurrentIndex(perso_data->j_VTS);
	aptitude_E[6]->setCurrentIndex(perso_data->j_ESQ);
	aptitude_E[7]->setCurrentIndex(perso_data->j_PRC);
	aptitude_E[8]->setCurrentIndex(perso_data->j_CHC);

	element_E[0]->setCurrentIndex(perso_data->j_attEle);
	for(int i=0 ; i<4 ; ++i)	element_E[i+1]->setCurrentIndex(perso_data->j_defEle[i]);

	mental_E[0]->setCurrentIndex(perso_data->j_attMtl);
	for(int i=0 ; i<4 ; ++i)	mental_E[i+1]->setCurrentIndex(perso_data->j_defMtl[i]);

	/* PAGE 6 */

	buildPage6();
}

void PersoEditor::savePage()
{
	if(id==255)	return;

	/* PAGE 1 */

	perso_data->exists = (inconnu6_E->value() << 3) | (lock2E->isChecked() << 2) | (lock1E->isChecked() << 1) | existsE->isChecked();

	const char *persoName = FF8Text::toByteArray(nameE->text(), jp).leftJustified(11, '\x00', true).append('\x00').constData();
	if(id == 0)			memcpy(&descData->squall, persoName, 12);
	else if(id == 4)	memcpy(&descData->linoa, persoName, 12);

	perso_data->current_HPs = current_HPsE->value();
	perso_data->HPs = HPs_E->value();
	perso_data->exp = expE->value();
	perso_data->VGR = vgr_E->value();
	perso_data->DFS = dfs_E->value();
	perso_data->MGI = mgi_E->value();
	perso_data->PSY = psy_E->value();
	perso_data->VTS = vts_E->value();
	perso_data->CHC = chc_E->value();
	perso_data->kills = kills_E->value();
	perso_data->KOs = kos_E->value();
	perso_data->ID = id_E->currentIndex();
	perso_data->weaponID = weapon_E->currentIndex();

	perso_data->status = 0;
	for(int i=0 ; i<8 ; ++i)
		perso_data->status |= status_list_E.at(i)->isChecked()<<i;

	perso_data->u1 = inconnu1_E->value();
	perso_data->u2 = inconnu2_E->value();
	perso_data->u3 = inconnu3_E->value();
	perso_data->u4 = inconnu4_E->value();
	perso_data->u5 = inconnu5_E->value();
	perso_data->alternative_model = alternativeE->value();

	/* PAGE 2 */

	for(int i=0 ; i<16 ; ++i)
	{
		perso_data->gfs = (poss_E.at(i)->isChecked() ? perso_data->gfs | (1 << i) : perso_data->gfs & ~(1 << i));
		perso_data->compatibility[i] = 6000-comp_E.at(i)->value();
	}

	/* PAGE 3 */

	for(int i=0 ; i<32 ; ++i)
	{
		perso_data->magies[i] = magie_E_model->item(i, 0)->data(Qt::UserRole+1).toInt() & 0xFF;
		perso_data->magies[i] |= (magie_E_model->item(i, 1)->text().toInt() & 0xFF) << 8;
	}

	/* PAGE 4 */

	perso_data->commands[0] = commande1_E->currentIndex();
	perso_data->commands[1] = commande2_E->currentIndex();
	perso_data->commands[2] = commande3_E->currentIndex();
	perso_data->abilities[0] = ability1_E->currentIndex();
	perso_data->abilities[1] = ability2_E->currentIndex();
	perso_data->abilities[2] = ability3_E->currentIndex();
	perso_data->abilities[3] = ability4_E->currentIndex();

	/* PAGE 5 */

	perso_data->j_HP = aptitude_E[0]->currentIndex();
	perso_data->j_VGR = aptitude_E[1]->currentIndex();
	perso_data->j_DFS = aptitude_E[2]->currentIndex();
	perso_data->j_MGI = aptitude_E[3]->currentIndex();
	perso_data->j_PSY = aptitude_E[4]->currentIndex();
	perso_data->j_VTS = aptitude_E[5]->currentIndex();
	perso_data->j_ESQ = aptitude_E[6]->currentIndex();
	perso_data->j_PRC = aptitude_E[7]->currentIndex();
	perso_data->j_CHC = aptitude_E[8]->currentIndex();

	perso_data->j_attEle = element_E[0]->currentIndex();
	for(int i=0 ; i<4 ; ++i)	perso_data->j_defEle[i] = element_E[i+1]->currentIndex();

	perso_data->j_attMtl = mental_E[0]->currentIndex();
	for(int i=0 ; i<4 ; ++i)	perso_data->j_defMtl[i] = mental_E[i+1]->currentIndex();

	/* PAGE 6 */

	switch(id)
	{
	case 0://Squall
		data->stats.renzokuken_auto = LB_E.first()->isChecked();
		data->stats.renzokuken_indicator = !LB_E.at(1)->isChecked() ? 128 : LBindicator_E->value();
		break;
	case 1://Zell
		data->limitb.zell = 0;
		for(int i=0 ; i<10 ; ++i)
			data->limitb.zell |= LB_E.at(i)->isChecked() << i;
		break;
	case 2://Irvine
		data->limitb.irvine = 0;
		for(int i=0 ; i<8 ; ++i)
			data->limitb.irvine |= LB_E.at(i)->isChecked() << i;
		break;
	case 3://Quistis
		data->limitb.quistis = 0;
		for(int i=0 ; i<16 ; ++i)
			data->limitb.quistis |= LB_E.at(i)->isChecked() << i;
		break;
	case 4://Linoa
		persoName = FF8Text::toByteArray(angel_E->text(), jp).leftJustified(11, '\x00', true).append('\x00').constData();
		memcpy(&descData->angelo, persoName, 12);
		data->limitb.angel_known = data->limitb.angel_completed = 0;
		for(int i=0 ; i<8 ; ++i)
		{
			data->limitb.angel_known |= (LB_E.at(i)->checkState() != Qt::Unchecked) << i;
			data->limitb.angel_completed |= (LB_E.at(i)->checkState() == Qt::Checked) << i;
			data->limitb.angel_pts[i] = linoaLB_E.at(i)->value();
		}
		data->stats.dream = (a_wing_enabledE->isChecked() << 5) | (angel_disabledE->isChecked() << 4) | (data->stats.dream & 0xCF);
		break;
	case 5://Selphie
		data->limitb.selphie = 0;
		for(int i=0 ; i<6 ; ++i)
			data->limitb.selphie |= LB_E.at(i)->isChecked() << i;
		break;
	}
}

void PersoEditor::exp_S(double value)
{
	int niv = ((quint32)value/1000)+1;
	if(niv>100)	niv = 100;
	if(nivE->value() != niv)
	{
		nivE->setValue(niv);
	}
}

void PersoEditor::niv_S(int value)
{
	qint32 exp = (value-1)*1000;
	if(exp > expE->value() || exp + 1000 <= expE->value())
	{
		expE->setValue(exp);
	}
}

void PersoEditor::updateCompLabels()
{
	for(int i=0 ; i<16 ; ++i)
	{
		compLabel_E.at(i)->setText(QString("(%1)").arg(comp_E.at(i)->value()/5));
	}
}

void PersoEditor::LBautoChange(int state)
{
	LBindicator_E->setEnabled(state == Qt::Unchecked && LB_E.at(1)->isChecked());
	LB_E.at(1)->setEnabled(state == Qt::Unchecked);
}

void PersoEditor::LBindicatorChange(int state)
{
	LBindicator_E->setEnabled(state == Qt::Checked);
}
