/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "TTriadEditor.h"

TTriadEditor::TTriadEditor(QWidget *parent) : QWidget(parent)
{
	// buildWidget();
	builded = false;
}

void TTriadEditor::load(MAIN *data)
{
	if(!builded)
		buildWidget();
	this->data = data;
	fillPage();
}

void TTriadEditor::updateMode()
{
}

void TTriadEditor::buildWidget()
{
	builded = true;
	QVBoxLayout *layout = new QVBoxLayout(this);

	tabWidget = new QTabWidget(this);
	tabWidget->addTab(buildPage1(), tr("Cartes"));
	tabWidget->addTab(buildPage2(), tr("Rgles"));

	layout->addWidget(tabWidget);
	layout->setContentsMargins(QMargins());
}

QWidget *TTriadEditor::buildPage1()
{
	QWidget *ret = new QWidget(this);

	QFont font;
	font.setPointSize(10);

	cardE_model = new QStandardItemModel(ret);
	cardE_view = new QTreeView(ret);
	cardE_view->setModel(cardE_model);
	cardE_view->setEditTriggers(QAbstractItemView::DoubleClicked);
	cardE_view->setFont(font);
	cardE_view->setItemDelegate(new SpinBoxDelegate(ret));
	cardE_view->setIndentation(0);
	cardE_view->setItemsExpandable(false);
	cardE_view->header()->setStretchLastSection(false);
	cardE_view->setUniformRowHeights(true);
	cardE_view->setAllColumnsShowFocus(true);

	cardE_model2 = new QStandardItemModel(ret);
	cardE_view2 = new QTreeView(ret);
	cardE_view2->setModel(cardE_model2);
	cardE_view2->setEditTriggers(QAbstractItemView::DoubleClicked);
	cardE_view2->setFont(font);
	cardE_view2->setItemDelegate(new SpinBoxDelegate(ret));
	cardE_view2->setIndentation(0);
	cardE_view2->setItemsExpandable(false);
	cardE_view2->header()->setStretchLastSection(false);
	cardE_view2->setUniformRowHeights(true);
	cardE_view2->setAllColumnsShowFocus(true);

	font.setPixelSize(10);
	QString selectAll = tr("Slectionner tout");
	QCheckBox *selectAll1 = new QCheckBox(selectAll, ret);
	QCheckBox *selectAll2 = new QCheckBox(selectAll, ret);
	selectAll1->setFont(font);
	selectAll2->setFont(font);
	QWidget *qtyAll = new QWidget;
	QPushButton *qtyAllOK = new QPushButton(tr("Tout"), qtyAll);
	QPushButton *squallAll = new QPushButton(tr("Squall"), ret);
	qtyAllOK->setFont(font);
	squallAll->setFont(font);
	qtyAllSpin = new QSpinBox(qtyAll);
	qtyAllSpin->setRange(0, 127);
	QHBoxLayout *qtyLayout = new QHBoxLayout(qtyAll);
	qtyLayout->addWidget(qtyAllSpin, 1);
	qtyLayout->addWidget(qtyAllOK);
	qtyLayout->setContentsMargins(QMargins());

	connect(selectAll1, SIGNAL(toggled(bool)), SLOT(selectAll1(bool)));
	connect(selectAll2, SIGNAL(toggled(bool)), SLOT(selectAll2(bool)));
	connect(qtyAllOK, SIGNAL(released()), SLOT(qtyAll()));
	connect(squallAll, SIGNAL(released()), SLOT(squallAll()));

	QGridLayout *layout = new QGridLayout(ret);
	layout->addWidget(new QLabel(tr("<b>Case coche :</b> carte connue<br/><b>Possesseur :</b> joueur possdant la carte rare, mettre Squall pour l'avoir dans votre jeu."), ret), 0, 0, 1, 4);
	layout->addWidget(cardE_view, 1, 0, 1, 2);
	layout->addWidget(selectAll1, 2, 0, Qt::AlignLeft);
	layout->addWidget(qtyAll, 2, 1, Qt::AlignRight);
	layout->addWidget(cardE_view2, 1, 2, 1, 2);
	layout->addWidget(selectAll2, 2, 2, Qt::AlignLeft);
	layout->addWidget(squallAll, 2, 3, Qt::AlignRight);

	return ret;
}

QWidget *TTriadEditor::buildPage2()
{
	QStringList regions;
	regions << tr("Balamb") << tr("Galbadia")
			<< tr("Trabia") << tr("Centra") << tr("Dollet")
			<< tr("Horizon") << tr("Lunar Gate") << tr("Esthar");

	QWidget *ret = new QWidget(this);

	ruleE_list = new QListWidget(ret);
	ruleE_list->addItems(QStringList() << regions << tr("Dernires rgles (1)") << tr("Dernires rgles (2)"));
	connect(ruleE_list, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), SLOT(fillRules(QListWidgetItem*,QListWidgetItem*)));


	QGroupBox *group1 = new QGroupBox(tr("Rgles"), ret);
	QVBoxLayout *group1L = new QVBoxLayout(group1);
	QStringList rules_names;
	rules_names << tr("Open") << tr("Identique") << tr("Plus") << tr("Alatoire") << tr("Mort subite") << tr("(Essai)") << tr("Mme mur") << tr("lmentaire");
	QCheckBox *rule;
	for(int i=0 ; i<8 ; ++i) {
		rule = new QCheckBox(rules_names.at(i), group1);
		group1L->addWidget(rule);
		rules.append(rule);
	}
	group1L->addStretch(1);

	group2 = new QGroupBox(tr("Rgle du vainqueur"), ret);
	QVBoxLayout *group2L = new QVBoxLayout(group2);

	QStringList traderules_names;
	traderules_names << tr("Aucune") << tr("One") << tr("Change") << tr("Direct") << tr("Totale") << "???" << "???" << "???";
	QRadioButton *trade_rule;
	for(int i=0 ; i<8 ; ++i) {
		trade_rule = new QRadioButton(traderules_names.at(i), group2);
		group2L->addWidget(trade_rule);
		trade_rules.append(trade_rule);
	}
	group2L->addStretch(1);

	QGroupBox *group3 = new QGroupBox(tr("Divers"), ret);
	QGridLayout *group3L = new QGridLayout(group3);

	last_regionE = new QComboBox(group3);
	last_regionE->addItem(QString::number(0));
	last_regionE->addItems(regions);
	for(int i=last_regionE->count() ; i<256 ; ++i)
		last_regionE->addItem(QString::number(i));

	last2_regionE = new QComboBox(group3);
	last2_regionE->addItem(QString::number(0));
	last2_regionE->addItems(regions);
	for(int i=last2_regionE->count() ; i<256 ; ++i)
		last2_regionE->addItem(QString::number(i));

	queenE = new QComboBox(group3);
	queenE->addItem(QString::number(0));
	queenE->addItems(QStringList() << tr("Balamb City") << tr("Deling City") << tr("Shumi village") << tr("Winhill") << tr("Dollet") << tr("Horizon") << tr("Lunar Gate") << tr("Esthar City"));
	for(int i=queenE->count() ; i<256 ; ++i)
		queenE->addItem(QString::number(i));

	traderatingE = new QSpinBox(group3);
	traderatingE->setRange(0, MAX_INT8);
	traderating_regionE = new QComboBox(group3);
	traderating_regionE->addItem(QString::number(0));
	traderating_regionE->addItems(regions);
	for(int i=traderating_regionE->count() ; i<256 ; ++i)
		traderating_regionE->addItem(QString::number(i));

	ttcountE = new QSpinBox(group3);
	ttcountE->setRange(0, MAX_INT16);

	ttdegenerationE = new QSpinBox(group3);
	ttdegenerationE->setRange(0, MAX_INT8);

	ttcardqueenquestE = new QComboBox(group3);
	ttcardqueenquestE->addItem(tr("MiniMog"));
	ttcardqueenquestE->addItem(tr("Tauros"));
	ttcardqueenquestE->addItem(tr("Chicobo"));
	ttcardqueenquestE->addItem(tr("Alexander"));
	ttcardqueenquestE->addItem(tr("Helltrain"));
	ttcardqueenquestE->addItem(tr("Toutes"));
	for(int i=ttcardqueenquestE->count() ; i<256 ; ++i)
		ttcardqueenquestE->addItem(QString::number(i));

	group3L->addWidget(new QLabel(tr("Dernires rgions visites :"),group3), 0, 0);
	group3L->addWidget(last_regionE, 0, 1);
	group3L->addWidget(last2_regionE, 0, 2);
	group3L->addWidget(new QLabel(tr("Emplacement reine des cartes :"),group3), 2, 0);
	group3L->addWidget(queenE, 2, 1);
	group3L->addWidget(new QLabel(tr("Indicateur rgle du vainqueur :"),group3), 3, 0);
	group3L->addWidget(traderatingE, 3, 1);
	group3L->addWidget(new QLabel(tr("Rgion :"),group3), 3, 2);
	group3L->addWidget(traderating_regionE, 3, 3);
	group3L->addWidget(new QLabel(tr("Dgnration :"),group3), 4, 0);
	group3L->addWidget(ttdegenerationE, 4, 1);
	group3L->addWidget(new QLabel(tr("Cartes cres :"),group3), 4, 2);
	group3L->addWidget(ttcardqueenquestE, 4, 3);
	group3L->addWidget(new QLabel(tr("Nombre de parties :"),group3), 5, 0);
	group3L->addWidget(ttcountE, 5, 1);

	QGridLayout *layout = new QGridLayout(ret);
	layout->addWidget(ruleE_list, 0, 0);
	layout->addWidget(group1, 0, 1);
	layout->addWidget(group2, 0, 2);
	layout->addWidget(group3, 1, 0, 1, 4);
	layout->setRowStretch(2, 1);
	layout->setColumnStretch(3, 1);

	return ret;
}

void TTriadEditor::fillPage()
{
	QStringList labels;
	QStandardItem *standardItem;

	cardE_model->clear();
	
	labels << tr("Nom") << tr("Qt");
	cardE_model->setHorizontalHeaderLabels(labels);
	
	for(quint8 i=0 ; i<77 ; ++i)
	{
		QList<QStandardItem *> items;
		standardItem = new QStandardItem(Data::cards.at(i));
		standardItem->setEditable(false);
		standardItem->setCheckable(true);
		standardItem->setCheckState(data->fieldVars.ttcards.cards[i] >> 7 ? Qt::Checked : Qt::Unchecked);
		standardItem->setIcon(QIcon(":/images/icons/carte.png"));
		items << standardItem;
		
		standardItem = new QStandardItem(QString::number(data->fieldVars.ttcards.cards[i] & 0x7F));
		standardItem->setData(3, Qt::UserRole);
		items << standardItem;
		
		cardE_model->appendRow(items);
	}
	
	cardE_view->header()->setResizeMode(0, QHeaderView::Stretch);
	cardE_view->header()->setResizeMode(1, QHeaderView::ResizeToContents);
	
	cardE_model2->clear();
	labels.replace(1, tr("Possesseur"));
	cardE_model2->setHorizontalHeaderLabels(labels);
	int locID;
	QString location;
	
	for(quint8 i=0 ; i<33 ; ++i)
	{
		QList<QStandardItem *> items;
		standardItem = new QStandardItem(Data::cards.at(77+i));
		standardItem->setEditable(false);
		standardItem->setCheckable(true);
		standardItem->setCheckState(data->fieldVars.ttcards.cards_rare[i/8] & (1 << (i%8)) ? Qt::Checked : Qt::Unchecked);
		standardItem->setIcon(QIcon(":/images/icons/carte.png"));
		items << standardItem;
		
		locID = Data::cardsLocation[data->fieldVars.ttcards.card_locations[i]];
		if(locID==-1)		location = tr("Utilise");
		else if(locID==-2)	location = tr("Squall");
		else				location = Data::locations.at(locID);
		standardItem = new QStandardItem(location);
		standardItem->setData(4, Qt::UserRole);
		standardItem->setData(data->fieldVars.ttcards.card_locations[i], Qt::UserRole+1);
		items << standardItem;
		
		cardE_model2->appendRow(items);
	}

	cardE_view2->header()->setResizeMode(0, QHeaderView::Stretch);
	cardE_view2->header()->setResizeMode(1, QHeaderView::ResizeToContents);

	ruleE_list->setCurrentRow(0);
	last_regionE->setCurrentIndex(data->fieldVars.ttrules.tt_lastregion[0]);
	last2_regionE->setCurrentIndex(data->fieldVars.ttrules.tt_lastregion[1]);
	queenE->setCurrentIndex(data->fieldVars.ttrules.tt_cardqueen_location);
	traderatingE->setValue(data->fieldVars.ttrules.tt_traderating);
	traderating_regionE->setCurrentIndex(data->fieldVars.ttrules.tt_traderating_region);
	ttcountE->setValue(data->fieldVars.ttcards.tt_victory_count);
	ttdegenerationE->setValue(data->fieldVars.ttrules.tt_degeneration);
	ttcardqueenquestE->setCurrentIndex(data->fieldVars.ttrules.tt_cardqueen_quest);
}

void TTriadEditor::fillRules(QListWidgetItem *, QListWidgetItem *previous)
{
	int id = ruleE_list->currentRow(), i;
	saveRules(ruleE_list->row(previous));
	quint8 tt_rules;

	if(id<8)
	{
		tt_rules = data->fieldVars.ttrules.tt_rules[id];
		for(i=0 ; i<8 ; ++i) {
			rules.at(i)->setChecked((tt_rules >> i) & 1);
		}

		tt_rules = data->fieldVars.ttrules.tt_traderules[id];
		for(i=0 ; i<8 ; ++i) {
			trade_rules.at(i)->setChecked(tt_rules == i);
		}
		group2->show();
	}
	else if(id<10)
	{
		tt_rules = data->fieldVars.ttrules.tt_lastrules[id-8];
		for(i=0 ; i<8 ; ++i) {
			rules.at(i)->setChecked((tt_rules >> i) & 1);
		}
		group2->hide();
	}
}

void TTriadEditor::savePage()
{
	quint8 value;
	quint8 exists;
	bool ok;
	for(quint8 i=0 ; i<77 ; ++i)
	{
		value = cardE_model->item(i, 1)->text().toUInt() & 0x7F;
		exists = (cardE_model->item(i, 0)->checkState() == Qt::Checked) << 7;
		data->fieldVars.ttcards.cards[i] = exists | value;
	}
	for(quint8 i=0 ; i<33 ; ++i)
	{
		data->fieldVars.ttcards.cards_rare[i/8] = cardE_model2->item(i, 0)->checkState() == Qt::Checked ? ( data->fieldVars.ttcards.cards_rare[i/8] | (1 << (i%8)) ) : ( data->fieldVars.ttcards.cards_rare[i/8] & ~(1 << (i%8)) );
		value = cardE_model2->item(i, 1)->data(Qt::UserRole+1).toUInt(&ok);
		if(ok)	data->fieldVars.ttcards.card_locations[i] = value;
	}
	saveRules(ruleE_list->currentRow());
	data->fieldVars.ttrules.tt_lastregion[0] = last_regionE->currentIndex();
	data->fieldVars.ttrules.tt_lastregion[1] = last2_regionE->currentIndex();
	data->fieldVars.ttrules.tt_cardqueen_location = queenE->currentIndex();
	data->fieldVars.ttrules.tt_traderating = traderatingE->value();
	data->fieldVars.ttrules.tt_traderating_region = traderating_regionE->currentIndex();
	data->fieldVars.ttcards.tt_victory_count = ttcountE->value();
	data->fieldVars.ttrules.tt_degeneration = ttdegenerationE->value();
	data->fieldVars.ttrules.tt_cardqueen_quest = ttcardqueenquestE->currentIndex();
}

void TTriadEditor::saveRules(int id)
{
	quint8 tt_rules;
	int i;

	if(id<0)	return;

	if(id<8)
	{
		tt_rules = 0;
		for(i=0 ; i<8 ; ++i) {
			tt_rules = tt_rules | (rules.at(i)->isChecked() << i);
		}

		data->fieldVars.ttrules.tt_rules[id] = tt_rules;

		tt_rules = 0;
		for(i=0 ; i<8 ; ++i) {
			if(trade_rules.at(i)->isChecked()) {
				tt_rules = i;
				break;
			}
		}
		// Change 3 bit only
		data->fieldVars.ttrules.tt_traderules[id] = (data->fieldVars.ttrules.tt_traderules[id] & 0xF8) | tt_rules;
	}
	else if(id<10)
	{
		tt_rules = 0;
		for(i=0 ; i<8 ; ++i) {
			tt_rules = tt_rules | (rules.at(i)->isChecked() << i);
		}

		data->fieldVars.ttrules.tt_lastrules[id-8] = tt_rules;
	}
}

void TTriadEditor::selectAll1(bool checked)
{
	for(quint8 i=0 ; i<77 ; ++i)
	{
		cardE_model->item(i)->setCheckState(checked ? Qt::Checked : Qt::Unchecked);
	}
}

void TTriadEditor::qtyAll()
{
	int qty = qtyAllSpin->value();
	for(quint8 i=0 ; i<77 ; ++i)
	{
		cardE_model->item(i, 1)->setText(QString::number(qty));
	}
}

void TTriadEditor::selectAll2(bool checked)
{
	for(quint8 i=0 ; i<33 ; ++i)
	{
		cardE_model2->item(i)->setCheckState(checked ? Qt::Checked : Qt::Unchecked);
	}
}

void TTriadEditor::squallAll()
{
	for(quint8 i=0 ; i<33 ; ++i)
	{
		cardE_model2->item(i, 1)->setData(240);
		cardE_model2->item(i, 1)->setText(tr("Squall"));
	}
}
