/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "Savecard.h"

Savecard::Savecard(QString chemin, QWidget *parent, bool slot)
	: QListWidget(parent), _ok(true), notify(true)
{
	setPalette(QPalette(QColor(0,0,0)));
	setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
	setSelectionMode(QAbstractItemView::NoSelection);
	connect(&fileWatcher, SIGNAL(fileChanged(QString)), SLOT(notifyFileChanged(QString)));
	
	if(slot)
	{
		_chemin = chemin%"/";
		_name = QString();
		_type = PCDIR;

		this->directory();
	}
	else
	{
		setPath(chemin);
		this->start = 0;
		QString extension;
		int separator;

		if((separator = _name.lastIndexOf(".")) != -1)
		{
			extension = _name.mid(separator+1).toLower();
		}
		else
		{
			_type = PC;
			if(!pc())	addSave(QByteArray());
			return;
		}

		if(extension == "mcr" || extension == "ddf" || extension == "mc"
		   || extension == "mcd"|| extension == "mci" || extension == "ps"
		   || extension == "psm")
		{
			_type = PS;
			_ok = ps();
		}
		else if(extension == "mem" || extension == "vgs")
		{
			_type = VGS;
			this->start = 64;
			_ok = ps();
		}
		else if(extension == "gme")
		{
			_type = GME;
			this->start = 3904;
			_ok = ps();
		}
		else if(extension == "vmp")
		{
			_type = VMP;
			this->start = 128;
			_ok = ps();
		}
		else if(extension == "psv")
		{
			_type = PSV;
			_ok = ps3();
		}
		else
		{
			QMessageBox::warning(this, tr("Erreur"), tr("Fichier de type inconnu"));
			_ok = false;
		}

		if(extension == "psv" || extension == "vmp")
			QMessageBox::information(this, tr("Sauvegarde hasardeuse"), tr("Le format %1 est protg, l'enregistrement sera partiel et risque de ne pas fonctionner.").arg(extension));
	}
}

Savecard::~Savecard()
{
	foreach(SaveData *save, saves)	delete save;
}

QString Savecard::chemin()
{
	return _chemin;
}

QString Savecard::name()
{
	return _name;
}

void Savecard::setPath(QString path)
{
	int separator = path.lastIndexOf("/")+1;
	if(separator!=-1)
	{
		_chemin = path.left(separator);
		_name = path.mid(separator);
	}
	else
	{
		_chemin = QString();
		_name = path;
	}
}

void Savecard::setName(QString name)
{
	_name = name;
}

bool Savecard::ok()
{
	return _ok;
}

quint8 Savecard::type()
{
	return _type;
}

void Savecard::notifyFileChanged(const QString &path)
{
	if(!notify)	return;
	notify = false;
	if(!QFile::exists(path))
	{
		QMessageBox::warning(this->parentWidget(), tr("Fichier supprim"), tr("Le fichier '%1' a t supprim par un programme externe !").arg(path));
	}
	else
	{
		QMessageBox::warning(this->parentWidget(), tr("Fichier modifi"), tr("Le fichier '%1' a t modifi par un programme externe.").arg(path));
	}
	notify = true;
}

void Savecard::moveCursor(int row)
{
	for(int i=0 ; i<row ; ++i) {
		((SaveWidget *)itemWidget(item(i)))->hideCursor();
	}
	
	for(int i=row+1 ; i<count() ; ++i) {
		((SaveWidget *)itemWidget(item(i)))->hideCursor();
	}
}

void Savecard::updateSaveWidget(int saveID, bool saved)
{
	((SaveWidget *)itemWidget(item(saveID)))->setSaved(saved);
	((SaveWidget *)itemWidget(item(saveID)))->repaintAll();// repaint all
}

void Savecard::updateSaveWidgets()
{
	for(int i=0 ; i<count() ; ++i)
		updateSaveWidget(i, false);
}

bool Savecard::ps()
{
	QFile fic(_chemin + _name);

	if(!fic.exists())
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Le fichier n'existe plus.\n%1").arg(_chemin + _name));
		return false;
	}
	if(!fic.open(QIODevice::ReadOnly))
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Le fichier est protg en lecture."));
		return false;
	}
	if(fic.size() < start+131072)//start+8192*16
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Fichier trop court"));
		return false;
	}

	fic.seek(start+128);
	QByteArray header = fic.read(1920);//(128*15)
	fic.seek(start+SAVE_SIZE);

	for(quint8 i=0 ; i<15 ; ++i)
		addSave(fic.read(SAVE_SIZE), header.mid(128*i, 127));

	if(fileWatcher.files().size()<30)
		fileWatcher.addPath(_chemin + _name);

	return true;
}

bool Savecard::ps3()
{
	QFile fic(_chemin + _name);

	if(!fic.exists())
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Le fichier n'existe plus.\n%1").arg(_chemin + _name));
		return false;
	}
	if(!fic.open(QIODevice::ReadOnly))
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Le fichier est protg en lecture."));
		return false;
	}
	if(fic.size() < 8324)
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Fichier invalide"));
		return false;
	}

	fic.seek(90);
	QByteArray header = fic.read(30);
	fic.seek(132);

	addSave(fic.read(SAVE_SIZE), header);

	if(fileWatcher.files().size()<30)
		fileWatcher.addPath(_chemin + _name);

	return true;
}

bool Savecard::pc()
{
	int tailleC;
	QFile f(_chemin + _name);

	if(!f.exists() || !f.open(QIODevice::ReadOnly))
		return false;

	f.read((char *)&tailleC, 4);
	if(tailleC != f.size()-4)		return false;

	addSave(LZS::decompress(f.readAll(), FF8SAVE_SIZE));

//	if(fileWatcher.files().size()<30)
//		fileWatcher.addPath(_chemin + _name);

	return true;
}

void Savecard::directory()
{
	QTime t;t.start();
	for(quint8 i=0 ; i<30 ; ++i)
	{
		_name = QString("save%1").arg(i+1, 2, 10, QChar('0'));
		if(!pc())	addSave();
	}
//	qDebug() << "time: " << t.elapsed();
}

void Savecard::addSave(QByteArray data, QByteArray header)
{
	SaveData *saveData = new SaveData(count(), data, header);
	saves.append(saveData);

	QListWidgetItem *item = new QListWidgetItem(this);
	item->setSizeHint(QSize(672, 106));
	item->setFlags(Qt::NoItemFlags);

	SaveWidget *saveWidget = new SaveWidget(saveData, this);
	setItemWidget(item, saveWidget);

	connect(saveWidget, SIGNAL(entered(int)), SLOT(moveCursor(int)));
	connect(saveWidget, SIGNAL(released(SaveData*)), parent(), SLOT(editView(SaveData*)));
	if(_type!=PC && _type!=PCDIR)
		connect(saveWidget, SIGNAL(changed()), SLOT(sauver()));
}

QList<SaveData *> Savecard::getSaves()
{
	return saves;
}

void Savecard::sauver(QString saveAs)
{
	QString path = saveAs.isEmpty() ? _chemin + _name : saveAs;
	QTemporaryFile temp;
	QFile fic(_chemin + _name);
//	qDebug() << (_chemin + _name) << saveAs << path;

	if(!fic.exists())
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Le fichier n'existe plus.\n%1").arg(_chemin + _name));
		return;
	}
	if(!fic.open(QIODevice::ReadOnly))
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Le fichier est protg en lecture.\n%1").arg(_chemin + _name));
		return;
	}
	if(!temp.open())
	{
		fic.close();
		QMessageBox::warning(this, tr("Erreur"), tr("Impossible de crer un fichier temporaire"));
		return;
	}

	if(_type==PSV)
	{
		temp.write(fic.read(132));
		temp.write(saves.first()->isFF8() ? saves.first()->sauver() : fic.read(SAVE_SIZE));
	}
	else if(_type != PC && _type != PCDIR)
	{
		int index = path.lastIndexOf('.');
		QString extension = index != -1 ? path.mid(index+1).toLower() : "";
		bool abort;
		quint8 i;
		SaveData *save;

		QByteArray head = header(&fic, extension, !saveAs.isEmpty(), &abort);
		if(abort) return;

		temp.write(head);

		temp.write("MC", 2);//MC
		temp.write(QByteArray(125,'\x00'));
		temp.putChar('\x0E');//xor byte

		fic.seek(start+128);

		for(i=0 ; i<15 ; ++i)
		{
			save = saves.at(i);

			if(save->hasMC_header())
			{
				temp.write(save->saveMC_header());//128
				fic.seek(fic.pos() + 128);
			}
			else
			{
				temp.write(fic.read(128));//Main header
			}
		}

		temp.write(fic.read(6144));//Padding (8192-16*128)

		for(i=0 ; i<15 ; ++i)
		{
			save = saves.at(i);

			if(save->isFF8())
			{
//				compare(fic.peek(FF8SAVE_SIZE), save->sauver());
				temp.write(save->sauver());
				fic.seek(fic.pos() + SAVE_SIZE);
			}
			else
			{
				temp.write(fic.read(SAVE_SIZE));
			}
		}
	}

	fic.close();

	bool readdPath = false;
	if(fileWatcher.files().contains(path))
	{
		readdPath = true;
		fileWatcher.removePath(path);
	}

	if(QFile::exists(path) && !QFile::remove(path))
	{
		temp.close();
		QMessageBox::warning(this, tr("Erreur"), tr("Impossible de supprimer le fichier !\n%1\nchec de la sauvegarde.\nVrifiez que le fichier n'est pas utilis par un autre programme.").arg(path));
		if(readdPath)	fileWatcher.addPath(path);
		return;
	}
	if(!temp.copy(path))
	{
		QMessageBox::warning(this, tr("Erreur"), tr("chec de la sauvegarde."));
	}
	if(readdPath)	fileWatcher.addPath(path);
}

void Savecard::sauverUn(qint8 id, QString path)
{
	if(path.isEmpty())
		path = _chemin + _name;

	QTemporaryFile temp;
	if(!temp.open())
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Impossible de crer un fichier temporaire"));
		return;
	}

	if(id==-1)	id = 0;
	QByteArray result = LZS::compress(saves.at(id)->sauver());
	int size = result.size();
	temp.write((char *)&size, 4);
	temp.write(result);

	bool readdPath = false;
	if(fileWatcher.files().contains(path))
	{
		readdPath = true;
		fileWatcher.removePath(path);
	}

	if(QFile::exists(path) && !QFile::remove(path))
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Impossible de supprimer le fichier !\n%1\nchec de la sauvegarde.\nVrifiez que le fichier n'est pas utilis par un autre programme.").arg(path));
		if(readdPath)	fileWatcher.addPath(path);
		return;
	}
	if(!temp.copy(path))
	{
		QMessageBox::warning(this, tr("Erreur"), tr("chec de la sauvegarde."));
	}
	if(readdPath)	fileWatcher.addPath(path);
}

void Savecard::sauver2PS(QList<int> ids, QString path)
{
	QTemporaryFile temp;
	SaveData *save, *firstSave = saves.first();
	quint8 i;

	if(!temp.open())
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Impossible de crer un fichier temporaire"));
		return;
	}

	int index = path.lastIndexOf('.');
	QString extension = index != -1 ? path.mid(index+1).toLower() : "";
	bool abort;

	QByteArray head = header(NULL, extension, true, &abort);
	if(abort)	return;
	temp.write(head);

	if(_type == PC || _type == PCDIR) {
		HeaderDialog dialog(firstSave, this, CREATEVIEW);

		if(dialog.exec()!=QDialog::Accepted) return;
	}

	QByteArray firstSaveMC_header = firstSave->saveMC_header();

	temp.write("MC", 2);//MC
	temp.write(QByteArray(125,'\x00'));
	temp.putChar('\x0E');//xor byte

	for(i=0 ; i<15 ; ++i)
	{
		// 128 bytes
		if(i>=ids.size())
		{
			temp.write(firstSaveMC_header);
		}
		else
		{
			save = saves.at(ids.at(i));
			save->setMC_header(firstSaveMC_header);
			save->setMC_header_id(i);
			temp.write(save->saveMC_header());
		}
	}

	temp.write(QByteArray(6144, '\x00'));//Padding

	for(i=0 ; i<15 ; ++i)
	{
		// 8192 bytes
		if(i>=ids.size() || !saves.at(ids.at(i))->isFF8())
		{
			temp.write(QByteArray(SAVE_SIZE, '\x00'));
		}
		else
		{
			temp.write(saves.at(ids.at(i))->sauver());
		}
	}

	bool readdPath = false;
	if(fileWatcher.files().contains(path))
	{
		readdPath = true;
		fileWatcher.removePath(path);
	}

	if(QFile::exists(path) && !QFile::remove(path))
	{
		QMessageBox::warning(this, tr("Erreur"), tr("Impossible de supprimer le fichier !\n%1\nchec de la sauvegarde.\nVrifiez que le fichier n'est pas utilis par un autre programme.").arg(path));
		if(readdPath)	fileWatcher.addPath(path);
		return;
	}
	if(!temp.copy(path))
	{
		QMessageBox::warning(this, tr("Erreur"), tr("chec de la sauvegarde."));
	}
	if(readdPath)	fileWatcher.addPath(path);
}

QByteArray Savecard::header(QFile *srcFile, QString extension, bool saveAs, bool *abort)
{
	*abort = false;

	if(extension == "mem" || extension == "vgs")
	{
		// header : 64 bytes
		if(_type==VGS)
		{
			return srcFile->read(64);
		}
		else
		{
			return QByteArray("\x56\x67\x73\x4d\x01\x00\x00\x00\x01\x00\x00\x00\x01\x00\x00\x00\x00\x02", 18).append(QByteArray(46, '\x00'));
		}
	}
	else if(extension == "gme")
	{
		QByteArray header;
		// header : 64 bytes
		if(_type==GME)
		{
			header = srcFile->read(64);
		}
		else
		{
			header = QByteArray("\x31\x32\x33\x2d\x34\x35\x36\x2d\x53\x54\x44\x00\x00\x00\x00\x00\x00\x00\x01\x00\x01\x4d\x51\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\x00\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", 64);
		}

		// description : 3840 bytes
		if(saveAs)
		{
			QString old_desc;
			if(_type==GME)
			{
				//srcFile->seek(64);
				old_desc = QString(srcFile->read(3840));
			}

			QByteArray new_desc = desc_gme(old_desc, abort);
			if(*abort)	return QByteArray();
			return header.append(new_desc);
		}
		else
		{
			//srcFile->seek(64);
			return header.append(srcFile->read(3840));
		}
	}
	else if(extension == "vmp")
	{
		// header : 128 bytes
		if(_type==VMP)
		{
			return srcFile->read(128);
		}
		else
		{
			//Unknown crc
			return QByteArray("\x00\x50\x4d\x56\x80", 5).append(QByteArray(123, '\x00'));
		}
	}
	else
		return QByteArray();
}

QByteArray Savecard::desc_gme(QString desc, bool *abort)
{
	QDialog dialog(this, Qt::Dialog | Qt::WindowCloseButtonHint);
	dialog.setWindowTitle(tr("Commentaire"));

	QTextEdit *textEdit = new QTextEdit(&dialog);
	textEdit->setPlainText(desc);

	QPushButton *ok = new QPushButton(tr("OK"), &dialog);
	QPushButton *cancel = new QPushButton(tr("Annuler"), &dialog);

	QGridLayout *layout = new QGridLayout(&dialog);
	layout->addWidget(textEdit, 0, 0, 1, 2);
	layout->addWidget(ok, 1, 0);
	layout->addWidget(cancel, 1, 1);

	connect(ok, SIGNAL(released()), &dialog, SLOT(accept()));
	connect(cancel, SIGNAL(released()), &dialog, SLOT(reject()));

	if(dialog.exec() != QDialog::Accepted) {
		*abort = true;
		return QByteArray();
	}
	*abort = false;

	return textEdit->toPlainText().toLatin1().leftJustified(3840, '\x00', true);
}

void Savecard::sauverDossier(quint8 i)
{
	_name = QString("save%1").arg(i+1, 2, 10, QChar('0'));
	sauverUn(i);
}

/* void Savecard::compare(QByteArray oldData, QByteArray newData)
{
	qDebug() << "Compare ----";
	for(int i=0 ; i<FF8SAVE_SIZE ; ++i) {
		switch(i) {
		case 0:
			qDebug() << "\tHeader";break;
		case 464:
			qDebug() << "\tGF";break;
		case 1552:
			qDebug() << "\tPerso";break;
		case 2768:
			qDebug() << "\tZZ1";break;
		case 3168:
			qDebug() << "\tConfig";break;
		case 3188:
			qDebug() << "\tDivers 1";break;
		case 3220:
			qDebug() << "\tLimit Break";break;
		case 3236:
			qDebug() << "\tObjets";break;
		case 3664:
			qDebug() << "\tStats";break;
		case 3808:
			qDebug() << "\tVar map";break;
		case 4080:
			qDebug() << "\tTriple Triad Rules";break;
		case 4106:
			qDebug() << "\tZZ2";break;
		case 5216:
			qDebug() << "\tTriple Triad Cards";break;
		case 5331:
			qDebug() << "\tChocobo";break;
		}

		if(oldData.at(i)!=newData.at(i)) {
			qDebug() << QString("Difference at %1: %2 (%3) | %4 (%5)").arg(i).arg((quint8)oldData.at(i),2,16,QChar('0')).arg((quint8)oldData.at(i),8,2,QChar('0')).arg((quint8)newData.at(i),2,16,QChar('0')).arg((quint8)newData.at(i),8,2,QChar('0'));
		}
	}
}*/
